"""

Python Desktop Server - News Aggregator for RSS feeds

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: AggregatorTool.py,v 1.59 2003/06/12 12:21:17 gb Exp $

import re
import md5
import sys
import time
import string
import traceback
import sgmllib
import htmlentitydefs

from base64 import encodestring, decodestring

try:
	from cStringIO import StringIO
except:
	from StringIO import StringIO

import PyDS.Tool

modelist = [
	('raw', _('raw XML source')),
	('cooked', _('parsed XML source')),
	('parsed', _('parsed and interpreted XML source'))
]

# patch sgmllib to find namespaced tags
sgmllib.tagfind = re.compile('[a-zA-Z][-_.a-zA-Z0-9:]*')

cdatare = re.compile(r'<\!\[CDATA\[(.*?)\]\]>',re.DOTALL)

class RSSParser(sgmllib.SGMLParser):

	def asAnnotatedXML(self):
		xml = '##darkcyan##<?xml version="1.0" encoding="iso-8859-1"?>##\n'
		xml += '##darkcyan##<rss version="2.0">##\n'
		for chan in self.channels:
			xml += '  ##blue##<channel>##\n'
			for el in ('title', 'link', 'description', 'managingeditor', 'webmaster'):
				if chan.has_key(el):
					(value, attrs) = chan[el]
					tag = '<' + el
					if attrs:
						alist = map(lambda a: '%s=%s' % (a[0], a[1]), attrs)
						tag += ' ' + string.join(alist, ' ')
					tag += '>'
					xml += '    ##blue##%s##%s##blue##</%s>##\n' % (tag, value, el)
			if chan.has_key('<skiphours>'):
				if chan['<skiphours>']:
					xml += '    ##blue##<skipHours>##\n'
					for hour in chan['<skiphours>']:
						xml += '      ##blue##<hour>%d</hour>##\n' % hour
					xml += '    ##blue##</skipHours>##\n'
			if chan.has_key('<items>'):
				for item in chan['<items>']:
					xml += '    ##green##<item>##\n'
					for el in ('title', 'link', 'description', 'guid', 'pubdate', 'comments', 'source', 'trackback:ping'):
						if item.has_key(el):
							(value, attrs) = item[el]
							tag = '<' + el
							if attrs:
								alist = map(lambda a: '%s="%s"' % (a, attrs[a]), attrs.keys())
								tag += ' ' + string.join(alist, ' ')
							if value:
								tag += '>'
								xml += '      ##green##%s##%s##green##</%s>##\n' % (tag, value, el)
							else:
								tag += '/>'
								xml += '      ##green##%s##\n' % tag
						if item.has_key('<image>'):
							xml += '      ##green##<image src="%s"/>##' % item['<image>'][0]
					xml += '    ##green##</item>##\n'
			xml += '  ##blue##</channel>##\n'
		xml += '##darkcyan##</rss>##'
		return xml

	def reset(self):
		sgmllib.SGMLParser.reset(self)
		self.channels = []
		self.currentTag = ''
		self.currentValue = ''
		self.currentAttrs = []
		self.currentStack = []
		self.currentAnnotatedSource = ''
		self.lastContentFormat = ''
		self.currentFeedFormat = 'RSS'
		self.taglist = [
			'title', 'link', 'description', 'guid', 'pubdate',
			'managingeditor', 'webmaster', 'comments', 'source',
			'trackback:ping',
		]
		self.tagtranslations = {
			'channeltitle': 'title',
			'channellink': 'link',
			'channeldescription': 'description',
			'text': 'description',
			'url': 'link',
			'linetext': 'title'
		}
		for k in self.tagtranslations.keys():
			self.taglist.append(k)
		self.ignorelist = (
			'rss', 'language', 'copyright', 'lastbuilddate', 'category',
			'generator', 'ttl', 'docs', 'image',
			'dc:publisher', 'dc:rights', 'dc:date', 'dc:format',
			'dc:subject', 'mp:keywords', 'dc:language', 'dc:type',
			'sy:updateperiod', 'sy:updatefrequency', 'sy:updatebase',
			'rdf', 'items', 'rdf:seq', 'rdf:li', 'rdf:bag',
			'content:items',
		)
		
	entitydefs = htmlentitydefs.entitydefs
	
	def _push(self, tag):
		self.currentStack.append((self.currentTag, self.currentValue, self.currentAttrs))
		self.currentTag = tag
		self.currentValue = ''
	
	def _pop(self, tag):
		while len(self.currentStack) and self.currentTag != tag:
			(self.currentTag, self.currentValue, self.currentAttrs) = self.currentStack.pop()
		if len(self.currentStack):
			(self.currentTag, self.currentValue, self.currentAttrs) = self.currentStack.pop()
	
	def _inItem(self):
		for el in self.currentStack:
			if self._logicalItem(el[0]):
				return 1
		return 0
	
	def _logicalChannel(self, where):
		if self.currentFeedFormat == 'scriptingnews':
			return where == 'header'
		else:
			return where == 'channel'
	
	def _logicalItem(self, where):
		return where == 'item'
	
	def _tagOpen(self, color, tag, attrs):
		if attrs:
			alist = map(lambda k: '%s="%s"' % k, attrs)
			self.currentAnnotatedSource += '##%s##<%s %s>##' % (
				color,
				tag,
				string.join(alist, ' '),
			)
		else:
			self.currentAnnotatedSource += '##%s##<%s>##' % (color, tag)
	
	def _tagClose(self, color, tag):
		self.currentAnnotatedSource += '##%s##</%s>##' % (color, tag)
	
	def start_scriptingnews(self, attrs):
		self.currentFeedFormat = 'scriptingnews'
		self._push('scriptingnews')
		self._tagOpen('blue', 'scriptingnews', attrs)
	
	def end_scriptingnews(self):
		self._pop('scriptingnews')
		self._tagClose('blue', 'scriptingnews')

	def start_header(self, attrs):
		if self.currentFeedFormat == 'scriptingnews':
			self._push('header')
			self.channels.append({'<items>':[]})
			self._tagOpen('blue', 'header', attrs)
		else:
			self.unknown_starttag('header', attrs)

	def end_header(self):
		if self.currentFeedFormat == 'scriptingnews':
			self._pop('header')
			self._tagClose('blue', 'header')
		else:
			self.unknown_endtag('header')

	def start_link(self, attrs):
		if self.currentFeedFormat == 'scriptingnews':
			self._push('link')
			self._tagOpen('green', 'link', attrs)
		else:
			self.unknown_starttag('link', attrs)

	def end_link(self):
		if self.currentFeedFormat == 'scriptingnews':
			self._pop('link')
			self._tagClose('green', 'link')
		else:
			self.unknown_endtag('link')

	def start_channel(self, attrs):
		self._push('channel')
		self.channels.append({'<items>':[]})
		self._tagOpen('blue', 'channel', attrs)
	
	def end_channel(self):
		self._pop('channel')
		self._tagClose('blue', 'channel')
	
	def start_skiphours(self, attrs):
		self._push('skiphours')
		self.channels[-1]['<skiphours>'] = []
		self._tagOpen('blue', 'skiphours', attrs)
	
	def end_skiphours(self):
		self._pop('skiphours')
		self._tagClose('blue', 'skiphours')
	
	def start_hour(self, attrs):
		self._push('hour')
		self._tagOpen('blue', 'hour', attrs)
	
	def end_hour(self):
		hour = int(self.currentValue)
		self.channels[-1]['<skiphours>'].append(hour)
		self._pop('hour')
		self._tagClose('blue', 'hour')
	
	def start_item(self, attrs):
		self._push('item')
		if not(len(self.channels)):
			self.channels.append({'<items>':[]})
		self.channels[-1]['<items>'].append({})
		self._tagOpen('green', 'item', attrs)
	
	def end_item(self):
		self._pop('item')
		self._tagClose('green', 'item')
	
	def start_trackback_ping(self, attrs):
		self._push('trackback:ping')
		if self._inItem():
			ahash = {}
			for (h, v) in attrs:
				ahash[h] = v
			self.channels[-1]['<items>'][-1]['trackback:ping'] = ('', ahash)
			self._tagOpen('green', 'trackback:ping', attrs)
		else:
			self._tagOpen('red', 'trackback:ping', attrs)
	
	def end_trackback_ping(self):
		self._pop('trackback:ping')
		if self._inItem():
			self._tagClose('green', 'trackback:ping')
		else:
			self._tagClose('red', 'trackback:ping')
	
	def start_content_item(self, attrs):
		self._push('content:item')
		self.currentAttrs = attrs
		if self._inItem():
			self._tagOpen('green', 'content:item', attrs)
		else:
			self._tagOpen('red', 'content:item', attrs)

	def end_content_item(self):
		attrs = self.currentAttrs
		self._pop('content:item')
		if self._inItem():
			if self.lastFormat[-14:] == '/JPEG/jfif.txt':
				self.lastFormat = ''
				for (a, v) in attrs:
					if a == 'rdf:about':
						self.channels[-1]['<items>'][-1]['<image>'] = (v, [])
			self._tagClose('green', 'content:item')
		else:
			self._tagClose('red', 'content:item')
		
	def start_content_format(self, attrs):
		for (a, v) in attrs:
			if a == 'rdf:resource':
				self.lastFormat = v
		if self.currentTag == 'content:item':
			self._tagOpen('green', 'content:format', attrs)
		else:
			self._tagOpen('red', 'content:format', attrs)

	def unknown_starttag(self, tag, attrs):
		pos = tag.find(':')
		if pos >= 0:
			newtag = tag.replace(':', '_')
			if hasattr(self, 'start_%s' % newtag):
				return getattr(self, 'start_%s' % newtag)(attrs)
		where = self.currentTag
		self._push(tag)
		self.currentAttrs = attrs
		if tag in self.taglist:
			if self._logicalChannel(where):
				self._tagOpen('blue', tag, attrs)
			elif self._logicalItem(where):
				self._tagOpen('green', tag, attrs)
			elif self.currentFeedFormat == 'scriptingnews' and self._inItem():
				if tag in ('url', 'linetext'):
					self._tagOpen('green', tag, attrs)
				else:
					self._tagOpen('red', tag, attrs)
			else:
				self._tagOpen('red', tag, attrs)
		elif tag in self.ignorelist:
			self._tagOpen('darkcyan', tag, attrs)
		else:
			self._tagOpen('darkmagenta', tag, attrs)
	
	def unknown_endtag(self, tag):
		pos = tag.find(':')
		if pos >= 0:
			newtag = tag.replace(':', '_')
			if hasattr(self, 'end_%s' % newtag):
				return getattr(self, 'end_%s' % newtag)()
		str = self.currentValue
		attrs = {}
		for (h, v) in self.currentAttrs:
			if attrs.has_key(h):
				if type(attrs[h]) == type([]):
					attrs[h].append(v)
				else:
					attrs[h] = [attrs[h], v]
			else:
				attrs[h] = v
		self._pop(tag)
		thetag = tag
		if self.tagtranslations.has_key(tag):
			thetag = self.tagtranslations[tag]
		if self._logicalItem(self.currentTag):
			if str:
				self.channels[-1]['<items>'][-1][thetag] = (str, attrs)
		elif self._logicalChannel(self.currentTag):
			if str:
				self.channels[-1][thetag] = (str, attrs)
		elif self.currentFeedFormat == 'scriptingnews' and self._inItem():
			if tag in ('url', 'linetext') and str:
				self.channels[-1]['<items>'][-1][thetag] = (str, attrs)
		if tag in self.taglist:
			if self._logicalChannel(self.currentTag):
				self._tagClose('blue', tag)
			elif self._logicalItem(self.currentTag):
				self._tagClose('green', tag)
			elif self.currentFeedFormat == 'scriptingnews' and self._inItem():
				if tag in ('url', 'linetext'):
					self._tagClose('green', tag)
				else:
					self._tagClose('red', tag)
			else:
				self._tagClose('red', tag)
		elif tag in self.ignorelist:
			self._tagClose('darkcyan', tag)
		else:
			self._tagClose('darkmagenta', tag)
	
	def handle_data(self, data):
		self.currentValue += data
		self.currentAnnotatedSource += data

# dummy class for channellist_html
class Feed:
	
	def __init__(self, title, link):
		self.title = title
		self.link = link
		self.count = 0
		self.seen = 0
		self.locked = 0

class AggregatorTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		try:
			self._acquire()
			self.feeds = self.db.getas(
				"feeds[url:S,title:S,link:S,desc:S,author:S,ready:I,message:S,lastrun:I,parsehook:S,skiphours:S]"
			).ordered(1)
			self.items = self.db.getas(
				"items[id:I,fetchtime:I,source:S,sourceurl:S,title:S,link:S,desc:S,guid:S,pubdate:S,othersource:S,othersourceurl:S,comments:S,trackbackurl:S,locked:I,seen:I]"
			).ordered(1)
			self.history = self.db.getas(
				"history[key:S,fetchtime:I,id:I,feedurl:S]"
			).ordered(1)
			self.prefs = self.db.getas(
				"prefs[maxid:I,interval:I,active:I,frontcount:I,showchannels:I]"
			)
			if len(self.prefs) == 0:
				self.prefs.append({'maxid':0})
			if self.prefs[0].interval < 60:
				self.prefs[0].interval = 60
			if self.prefs[0].frontcount == 0:
				self.prefs[0].frontcount = 100
			self._commit()
		finally: self._release()
	
	def _backup(self):
		def _selectlocked(row):
			return row.locked

		l = self._backupTable('feeds')
		l.extend(self._backupTable('prefs'))
		l.extend(self._backupTable('items', selectfn=_selectlocked))
		l.extend(self._backupTable('history'))
		return l

	def _initthread(self, server):
		while server.isRunning():
			time.sleep(10)
			todo = []
			now = time.time()
			for feed in self.getFeeds():
				if feed['lastrun'] + 60*self.getPref('interval') < now and self.getPref('active'):
					hour = time.gmtime(now)[3]
					if not(hour in feed['skiphours']):
						todo.append(feed['url'])
			if len(todo):
				for url in todo:
					if server.isRunning():
						self.updateFeed(url)
				self.logVerbose(_('work done, going to sleep again'))
	
	def _search(self, query):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			query = query.lower()
			liste = []
			idx = len(self.items)-1
			for item in self.items:
				if item.title.lower().find(query) >= 0 or item.desc.lower().find(query) >= 0:
					desc = macros.stripHTML(item.desc)
					if len(desc) > 200:
						desc = desc[:200] + ' ...'
					liste.append({
						'title': item.title,
						'link': self.getUrl(start=str(idx)),
						'description': desc,
						'guid': item.guid or item.link
					})
				idx -= 1
			liste.reverse()
			return liste
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d feeds in the database</li>') % (
				len(self.feeds),
			)
			html += _('<li>There are %d news items in the database</li>') % (
				len(self.items),
			)
			html += _('<li>There are %d news items in the history</li>') % (
				len(self.history),
			)
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
	def _prepareRSS(self, xml):
		macros = self.getToolByNamespace('macros')
		cdata = '<![CDATA['
		start = 0
		accum = StringIO()
		pos = xml.find(cdata, start)
		while pos >= 0:
			accum.write(xml[start:pos])
			start = pos+len(cdata)
			end = xml.find(']]', start)
			if end >= 0:
				accum.write(macros.quoteXML(xml[start:end]))
				start = end+2
				pos = xml.find(cdata, start)
			else:
				pos = -1
		accum.write(xml[start:])
		return accum.getvalue()
	
	def _parseRSS(self, xml):
		p = RSSParser()
		p.feed(xml)
		return p
	
	def getPreferencePanels(self):
		return [
			['aggregator_panel', _('Aggregator'),
				_('Here you can configure your aggregators parameters')]
		]
	
	def aggregator_panel(self, req):
		try:
			self._acquire()
			html = '<p>%s' % _('Here you can configure your aggregator')
			html += '<form action="%s" method="POST">' % req.getUrl('aggregator_redir', tool='aggregator')
			html += '<table border=0 width="100%">'
			html += '<tr><td>%s</td><td>' % _('aggreagator interval')
			html += '<input name="interval" size="5" value="%s">' % (
				self.prefs[0].interval
			)
			html += '</td></tr>'
			html += '<tr><td>%s</td><td>' % _('show how many news')
			html += '<input name="frontcount" size="5" value="%s">' % (
				self.prefs[0].frontcount
			)
			html += '</td></tr>'
			html += '<tr><td>%s</td><td>' % _('aggreagator active')
			html += '<select name="active">'
			if self.prefs[0].active:
				html += '<option value="1" selected>%s' % _('yes')
				html += '<option value="0">%s' % _('no')
			else:
				html += '<option value="1">%s' % _('yes')
				html += '<option value="0" selected>%s' % _('no')
			html += '</select>'
			html += '</td></tr>'
			html += '<tr><td>%s</td><td>' % _('index page')
			html += '<select name="showchannels">'
			if self.prefs[0].showchannels:
				html += '<option value="1" selected>%s' % _('show channel list')
				html += '<option value="0">%s' % _('show all items')
			else:
				html += '<option value="1">%s' % _('show channel list')
				html += '<option value="0" selected>%s' % _('show all items')
			html += '</select>'
			html += '</td></tr>'
			html += '</table>'
			html += '<input type=submit name="AggregatorSubmit" value="%s">' % _('save')
			html += '</form>'
			return html
		finally: self._release()
	
	def aggregator_redir(self, req):
		try:
			self._acquire()
			self.prefs[0].interval = int(req.getFirstValue('interval'))
			if self.prefs[0].interval < 30:
				self.prefs[0].interval = 30
			self.prefs[0].active = int(req.getFirstValue('active'))
			self.prefs[0].showchannels = int(req.getFirstValue('showchannels'))
			self.prefs[0].frontcount = int(req.getFirstValue('frontcount'))
			if self.prefs[0].frontcount < 10:
				self.prefs[0].frontcount = 10
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='aggregator_panel',
				fortool='aggregator'
			)
		finally: self._release()
	
	def _navigatePages(self, starting, numitems, allitems, source, sourceurl):
		html = ''
		if starting or starting+numitems < allitems:
			html += '<p align="right">'
			if starting > 0:
				newstart = starting-self.prefs[0].frontcount
				if newstart < 0:
					newstart = 0
				if sourceurl:
					html += '<a href="%s">%s</a>' % (
						self.getUrl('itemlist_html', start=newstart, source=source, sourceurl=sourceurl),
						_('%d newer items') % starting
					)
				else:
					html += '<a href="%s">%s</a>' % (
						self.getUrl(start=newstart),
						_('%d newer items') % starting
					)
			if starting+numitems < allitems:
				if starting > 0:
					html += ' | '
				newstart = starting+self.prefs[0].frontcount
				ctitems = allitems - newstart
				if sourceurl:
					html += '<a href="%s">%s</a>' % (
						self.getUrl('itemlist_html', start=newstart, source=source, sourceurl=sourceurl),
						_('%d older items') % ctitems
					)
				else:
					html += '<a href="%s">%s</a>' % (
						self.getUrl(start=newstart),
						_('%d older items') % ctitems
					)
		return html
	
	def itemlist_html(self, req):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			weblog = self.getToolByNamespace('weblog')
			starting = int(req.getFirstValue('start',0))
			source = req.getFirstValue('source', '')
			sourceurl = req.getFirstValue('sourceurl', '')
			itemlist = self.items
			if sourceurl:
				itemlist = self.items.select({'sourceurl':sourceurl})
			allitems = len(itemlist)
			if starting > allitems:
				if allitems > self.prefs[0].frontcount:
					starting = 0
				else:
					starting = allitems - self.prefs[0].frontcount
			numitems = allitems - starting
			if numitems > self.prefs[0].frontcount:
				numitems = self.prefs[0].frontcount
			html = ''
			if sourceurl:
				html = '<p class="whitebox">%s' % (_('%d news items from "%s"') % (
					numitems,
					(source or sourceurl)
				))
			else:
				html = '<p class="whitebox">%s' % (_('%d news items from your %d newsfeeds') % (
					numitems,
					len(self.feeds)
				))
			if sourceurl:
				html += '<p><a href="%s">%s</a><p>' % (
					self.getUrl('channellist_html'),
					_('Click here to go back to the news overview')
				)
			html += '<p><a href="%s">%s</a><p>' % (
				self.getUrl('overview_html'),
				_('Click here to configure your newsfeeds')
			)
			lastdate = ''
			lastsource = ''
			pageids = []
			html += self._navigatePages(starting, numitems, allitems, source, sourceurl)
			if numitems:
				for idx in range(starting+1,starting+numitems+1):
					item = itemlist[-idx]
					pageids.append(item.id)
					thisdate = macros.Today(now=item.fetchtime)
					if not(lastdate):
						html += '<table border=0 width="100%">'
					if thisdate != lastdate:
						if lastdate:
							html += '</td></tr>'
						html += '<tr><td class="whitebox">'
						html += '<p>%s' % thisdate
					if not(lastsource) or lastsource != item.source:
						if item.sourceurl:
							html += '<h3><a target="_blank" href="%s">%s <span class="smallfont">(%s)</span></a></h3>' % (
								item.sourceurl,
								item.source,
								macros.Today(now=item.fetchtime, withtime=1)
							)
						else:
							html += '<h3>%s</h3>' % item.source
					newimg = ''
					html += '<a name="item%s"></a>' % item.id
					if not(item.seen):
						newimg = '<img src="/images/new.gif" alt="%s">' % _('this item is new')
						item.seen = 1
						self._commit()
					if item.title:
						if item.link:
							html += '<p>%s<b><a target="_blank" href="%s">%s</a></b>' % (newimg, item.link, item.title)
							newimg = ''
						else:
							html += '<p>%s<b>%s</b>' % (newimg, item.title)
							newimg = ''
					elif item.link:
						html += '<p>%s<b><a target="_blank" href="%s">%s</a></b>' % (newimg, item.link, item.link)
						newimg = ''
					html += '<table border=0 width="100%"><tr><td width="99%">'
					html += '<p>'
					if newimg:
						html += newimg
					html += item.desc
					html += '<p align="right" class="smallfont">'
					if item.comments:
						html += '&nbsp;&nbsp;'
						html += '<a href="%s" %s title="%s">%s</a>' % (
							item.comments,
							macros.onClick('comments'),
							_('Click here to comment this post'),
							_('comment'),
						)
					if item.trackbackurl:
						html += '&nbsp;&nbsp;'
						html += '<a href="%s" %s title="%s">%s</a>' % (
							item.trackbackurl,
							macros.onClick('trackbacks'),
							_('Click here to see trackbacks'),
							_('trackback'),
						)
					if item.guid:
						html += '&nbsp;&nbsp;'
						html += '<a target="_blank" href="%s" title="%s">#</a>' % (
							item.guid,
							_('Permalink to this posting')
						)
					if item.pubdate:
						html += '&nbsp;&nbsp;'
						html += item.pubdate
					if item.othersource and item.othersourceurl:
						html += '<p align="right">%s: <a target="_blank" href="%s">%s</a>' % (
							_('source'),
							item.othersourceurl,
							item.othersource
						)
					elif item.othersource:
						html += '<p align="right">%s: %s' % (
							_('source'),
							item.othersource
						)
					html += '</td><td valign="bottom"><form target="_blank" action="%s" method="POST" title="%s">' % (
						weblog.getUrl(),
						_('click here to post this entry')
					)
					html += '<input type=hidden name="ptitle" value="%s">' % macros.quoteXML(item.title)
					html += '<input type=hidden name="plink" value="%s">' % (item.link or item.guid)
					html += '<input type=hidden name="psource" value="%s">' % macros.quoteXML(item.source)
					html += '<input type=hidden name="psourceurl" value="%s">' % item.sourceurl
					html += '<input type=hidden name="ptext" value="%s">' % macros.quoteXML(item.desc)
					html += '<input type=hidden name="ptburl" value="%s">' % item.trackbackurl
					html += '<input type=image src="/images/blog.png" alt="%s">' % (
						_('click here to post this entry')
					)
					html += '</form>'
					msg = _('click here to protect this entry from deletion')
					img = 'openlock.gif'
					if item.locked:
						msg = _('click here to unlock this entry to allow deletion')
						img = 'closedlock.gif'
					html += '<form action="%s" method="POST" title="%s">' % (
						req.getUrl('lock_redir'),
						msg
					)
					html += '<input type="hidden" name="pid" value="%s">' % item.id
					html += '<input type="hidden" name="start" value="%s">' % starting
					html += '<input type="image" src="/images/%s" alt="%s">' % (img, msg)
					if source: html += '<input type=hidden name="source" value="%s">' % source
					if sourceurl: html += '<input type=hidden name="sourceurl" value="%s">' % sourceurl
					html += '</form></td></tr></table>'
					lastdate = thisdate
					lastsource = item.source
				if lastdate:
					html += '</td></tr>'
					html += '</table>'
				html += self._navigatePages(starting, numitems, allitems, source, sourceurl)
				if sourceurl:
					html += '<p><a href="%s">%s</a><p>' % (
						self.getUrl('channellist_html'),
						_('Click here to go back to the news overview')
					)
				if len(pageids):
					html += '<form action="%s" method="POST">' % self.getUrl('purge_redir')
					for id in pageids:
						html += '<input type="hidden" name="id" value="%s">' % id
					html += '<input type="hidden" name="start" value="%s">' % starting
					if source: html += '<input type=hidden name="source" value="%s">' % source
					if sourceurl: html += '<input type=hidden name="sourceurl" value="%s">' % sourceurl
					html += '<input type="submit" name="PurgeSubmit" value="%s">' % (
						_('Purge this page of news')
					)
					html += '</form>'
			if sourceurl:
				req.setLocalValue('title', (source or sourceurl))
			else:
				req.setLocalValue('title', _('Aggregator'))
			return req.renderPage('BaseTemplate', body=html)
		finally: self._release()

	# this method returns the XML url of a feed. You can identify
	# an RSS feed by this URL
	def getFeedUrl(self, title, link):
		try:
			self._acquire()
			res = self.feeds.select({
				'title': title,
				'link': link
			})
			if len(res):
				return res[0].url
			else:
				raise IndexError((title, link))
		finally: self._release()
	
	def channellist_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = '<p><a href="%s">%s</a><p>' % (
			self.getUrl('overview_html'),
			_('Click here to configure your newsfeeds')
		)
		feeds = {}
		try:
			self._acquire()
			for feed in self.feeds:
				if feed.link and feed.link[-1] == '\n':
					feed.link = feed.link[:-1]
					self._commit()
				feeds[feed.link] = Feed(feed.title, feed.link)
			for item in self.items:
				if item.sourceurl and item.sourceurl[-1] == '\n':
					item.sourceurl = item.sourceurl[:-1]
					self._commit()
				if not(feeds.has_key(item.sourceurl)):
					feeds[item.sourceurl] = Feed(item.source, item.sourceurl)
				if feeds.has_key(item.sourceurl):
					feed = feeds[item.sourceurl]
					feed.count += 1
					if item.seen:
						feed.seen += 1
					if item.locked:
						feed.locked += 1
		finally: self._release()
		if len(feeds.keys()):
			html += '<table border=0 width="100%">'
			keylist = feeds.keys()
			keylist.sort()
			for feedkey in keylist:
				feed = feeds[feedkey]
				if feed.count:
					html += '<tr>'
					html += '<td colspan=2 class="whitebox">'
					feedlink = req.getUrl('itemlist_html', source=feed.title, sourceurl=feed.link)
					if feed.seen <> feed.count:
						html += '<b><a href="%s">%s</a></b>' % (
							feedlink, feed.title
						)
					else:
						html += '<a href="%s">%s</a>' % (
							feedlink, feed.title
						)
					html += '<p class="smallfont" align="right">%d %s' % (
						feed.count, _('items')
					)
					if feed.seen <> feed.count:
						html += ', <b>%d %s</b>' % (
							feed.count - feed.seen,
							_('new'),
						)
						html += ', %d %s' % (
							feed.locked,
							_('locked')
						)
					html += '</td></tr>'
			html += '</table>'
		req.setLocalValue('title', _('Overview of feeds with items'))
		return req.renderPage('BaseTemplate', body=html)
	
	def index_html(self, req):
		if self.getPref('showchannels'):
			return self.channellist_html(req)
		else:
			return self.itemlist_html(req)
	
	def purge_redir(self, req):
		try:
			self._acquire()
			pageids = req.getAllValues('id')
			starting = req.getFirstValue('start', 0)
			source = req.getFirstValue('source', '')
			sourceurl = req.getFirstValue('sourceurl', '')
			todel = []
			for id in pageids:
				(idx, found) = self.items.locate({'id':int(id)})
				if found:
					if not(self.items[idx].locked):
						todel.append(idx)
			todel.sort()
			todel.reverse()
			for idx in todel:
				self.items.delete(idx)
			self._commit()
			if sourceurl:
				itemlist = self.items.select({'sourceurl':sourceurl})
				if len(itemlist) == 0:
					return self.getUrl()
				elif starting:
					return self.getUrl('itemlist_html', start=starting, source=source, sourceurl=sourceurl)
				else:
					return self.getUrl('itemlist_html', source=source, sourceurl=sourceurl)
			else:
				if starting:
					return self.getUrl(start=starting)
				else:
					return self.getUrl()
		finally: self._release()
	
	def lock_redir(self, req):
		try:
			self._acquire()
			pid = req.getFirstValue('pid')
			starting = req.getFirstValue('start', 0)
			source = req.getFirstValue('source', '')
			sourceurl = req.getFirstValue('sourceurl', '')
			(idx, found) = self.items.locate({'id':int(pid)})
			if found:
				self.items[idx].locked = not(self.items[idx].locked)
				self._commit()
			if sourceurl:
				if starting:
					return req.getUrl('itemlist_html', fragment='item%s' % pid, start=starting, source=source, sourceurl=sourceurl)
				else:
					return req.getUrl('itemlist_html', fragment='item%s' % pid, source=source, sourceurl=sourceurl)
			else:
				if starting:
					return req.getUrl(fragment='item%s' % pid, start=starting)
				else:
					return req.getUrl(fragment='item%s' % pid)
		finally: self._release()
	
	def purgehistory_redir(self, req):
		self.purgeHistory()
		return self.getUrl('overview_html')

	def overview_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = '<p><a href="%s">%s</a><p>' % (
			self.getUrl(),
			_('Click here to go back to the news overview')
		)
		html += '<p><a href="%s">%s</a><p>' % (
			self.getUrl('purgehistory_redir'),
			_('Click here to purge your history of already seen articles')
		)
		html += '<form action="%s" method="POST">' % req.getUrl('addfeed_redir')
		html += '<input name="url" value="http://" size=40>&nbsp;'
		html += '<input type=submit name="AggregatorAdd" value="%s">' % _('Add')
		html += '</form>'
		if len(self.feeds):
			html += '<p><form action="%s" method="POST">' % req.getUrl('deletefeed_redir')
			html += '<table border=0 width="100%">'
			for feed in self.feeds.sort(self.feeds.title, self.feeds.url):
				html += '<tr>'
				html += '<td><input type=checkbox name="feed" value="%s"></td>' % feed.url
				html += '<td colspan=2 class="whiteboxsmall">'
				if feed.title and feed.desc:
					html += '<b><a target="_blank" href="%s">%s</a>: </b> %s' % (
						feed.link, feed.title, feed.desc
					)
				elif feed.title:
					html += '<b><a target="_blank" href="%s">%s</a></b>' % (
						feed.link, feed.title
					)
				else:
					html += '<b><a target="_blank" href="%s">%s</a></b>' % (
						feed.link, feed.url
					)
				html += '<p align="right">%s: %s</p>' % (
					_('last update'),
					macros.Today(now=feed.lastrun, withtime=1)
				)
				html += '</td><td><table border=0>'
				html += '<tr><td align="center"><a href="%s">' % (
					self.getUrl('show_html', url=encodestring(feed.url)),
				)
				html += '<img src="/images/miniXmlButton.gif" border=0></a></td></tr>' 
				if feed.author:
					html += '<tr><td align="center"><a href="mailto:%s">' % (
						feed.author,
					)
					html += '<img src="/images/mailto.gif" border=0></a></td></tr>' 
				html += '</table></td>'
				html += '</tr>'
				if feed.message:
					if feed.ready:
						html += '<tr><td colspan=3 class="infomsg">%s</td></tr>' % feed.message
					else:
						html += '<tr><td colspan=3 class="errormsg">%s</td></tr>' % feed.message
			html += '</table>'
			html += '<p><input type=submit name="AggregatorDelete" value="%s">' % _('Delete')
			html += ' <input type=submit name="AggregatorUpdate" value="%s">' % _('Update')
			html += ' <input type=submit name="AggregatorHistoryDelete" value="%s">' % _('Purge History')
			html += '</form>'
		req.setLocalValue('title', _("Add or delete RSS feeds"))
		return req.renderPage('BaseTemplate', body=html)
	
	def show_html(self, req):
		downstream = self.getToolByNamespace('downstream')
		macros =self.getToolByNamespace('macros')
		url = decodestring(req.getFirstValue('url'))
		mode = req.getFirstValue('mode', 'cooked')
		feed = self.getFeed(url)
		title = feed.get('title')
		hook = feed.get('parsehook')
		if not(title):
			title = feed.get('url')
		html = '<p><a href="%s">%s</a>' % (
			self.getUrl('overview_html'),
			_('Back to the overview page')
		)
		desc = feed.get('description')
		if title and desc:
			html += '<p class="whiteboxsmall">%s: %s' % (title, desc)
		elif title or desc:
			html += '<p class="whiteboxsmall">%s' % (desc or title)
		html += '<p class="smallfont"><b>URL:</b> %s' % url
		if feed.get('skiphours', []):
			html += '<p class="smallfont"><b>%s:</b> %s' % (
				_('skip hours (GMT)'),
				string.join(map(lambda i: str(i), feed['skiphours']), ', ')
			)
		html += '<p><form action="%s" method="POST">' % (
			self.getUrl('hook_redir'),
		)
		html += '<input type=hidden name=url value="%s"> ' % encodestring(url)
		html += '<select name="hook">'
		html += '<option value="">%s' % _('no macro selected')
		for macro in macros._getMacros():
			if hook == macro['name']:
				html += '<option value="%s" selected>%s' % (
					macro['name'],
					macro['name']
				)
			else:
				html += '<option value="%s">%s' % (
					macro['name'],
					macro['name']
				)
		html += '</select> '
		html += '<input type=submit name="HookSubmit" value="%s">' % _('save')
		html += '</form>'
		html += '<p><form action="%s" method="GET">' % (
			self.getUrl('show_html'),
		)
		html += '<input type=hidden name=url value="%s"> ' % encodestring(url)
		html += '<select name="mode">'
		for m in modelist:
			if mode == m[0]:
				html += '<option value="%s" selected>%s' % (
					m[0],
					m[1]
				)
			else:
				html += '<option value="%s">%s' % (
					m[0],
					m[1]
				)
		html += '</select> '
		html += '<input type=submit name="ModeSubmit" value="%s">' % _('switch format')
		html += '</form>'
		shownakedsource = 0
		xml = ''
		try:
			fp = downstream.download(url)
			source = fp.read()
			nsource = source
			source = re.sub('>\s*<', '>\n<', source, re.DOTALL)
			xml = self._prepareRSS(source)
			p = self._parseRSS(xml)
			if mode == 'raw':
				source = macros.quoteXML(xml)
			elif mode == 'cooked':
				source = macros.quoteXML(p.currentAnnotatedSource)
			elif mode == 'parsed':
				source = macros.quoteXML(p.asAnnotatedXML())
			else:
				source = str(dir(p))
			source = re.sub('##(.*?)##(.*?)##',
				lambda match: '<font color="%s"><b>%s</b></font>' % (match.group(1), match.group(2)),
				source
			)
			fp.close()
		except:
			(e, d, tb) = sys.exc_info()
			source = _('Exception "%s: %s"') % (e, d)
			for row in traceback.extract_tb(tb):
				(file, line, function, src) = row
				source += '<br>%s[%s] in %s' % (file, line, function)
			shownakedsource = 1
		html += '<pre class="whiteboxsmall">%s</pre>' % source
		if shownakedsource:
			html += '<pre class="whiteboxsmall">%s</pre>' % macros.quoteXML(xml)
		req.setLocalValue('title', 'Aggregator Feed Source')
		return req.renderPage('BaseTemplate', body=html, withcalendar=0)
	
	def hook_redir(self, req):
		try:
			self._acquire()
			url = decodestring(req.getFirstValue('url'))
			hook = req.getFirstValue('hook')
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				feed = self.feeds[idx]
				feed.parsehook = hook
				self._commit()
				return req.getUrl('show_html', url=encodestring(url))
			else:
				return req.getUrl('show_html', url=encodestring(url), errmsg=_("can't find feed with URL %s") % url)
		finally: self._release()
		
	def addfeed_redir(self, req):
		url = req.getFirstValue('url')
		if url and url != 'http://':
			self.addFeed(url)
		return req.getUrl('overview_html')
	
	def deletefeed_redir(self, req):
		feeds = req.getAllValues('feed')
		for feed in feeds:
			if req.getFirstValue('AggregatorDelete'):
				self.deleteFeed(feed)
			elif req.getFirstValue('AggregatorUpdate'):
				self.updateFeed(feed)
			elif req.getFirstValue('AggregatorHistoryDelete'):
				self.purgeHistory(feed)
		return req.getUrl('overview_html')
		
	def _translateToHash(self, feed):
		hlist = feed.skiphours.split(':')
		skiphours = []
		for hour in hlist:
			try: skiphours.append(int(hour))
			except: pass
		return {
			'url':feed.url,
			'title': feed.title,
			'link': feed.link,					
			'description': feed.desc,
			'author': feed.author,
			'message': feed.message,
			'ready': feed.ready,
			'parsehook': feed.parsehook,
			'lastrun': feed.lastrun,
			'skiphours': skiphours
		}
		
	# this method returns a list of feeds.
	def getFeeds(self):
		try:
			self._acquire()
			res = []
			for feed in self.feeds:
				res.append(self._translateToHash(feed))
			return res
		finally: self._release()
	
	# this method returns a dictionary with data for the feed
	# that has the given url
	def getFeed(self, url):
		try:
			self._acquire()
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				feed = self.feeds[idx]
				return self._translateToHash(feed)
			else:
				return {}
		finally: self._release()
		
	# this method adds a feed to your aggregator
	def addFeed(self, url):
		try:
			self._acquire()
			ready = 0
			message = _('This feed was not yet pulled from the internet')
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				return (self.feeds[idx].ready, _('this feed was already subscribed'))
			self.feeds.append({
				'url':url,
				'title':'',
				'link':url,
				'desc':'',
				'author':'',
				'ready':ready,
				'message':message
			})
			self._commit()
			return self.updateFeed(url)
		finally: self._release()
	
	def _setFeedData(self, url, feedtitle, feedlink, feeddesc, feedauthor, skiphours):
		try:
			self._acquire()
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				feed = self.feeds[idx]
				feed.title = feedtitle
				feed.link = feedlink
				feed.desc = feeddesc
				feed.author = feedauthor
				feed.skiphours = string.join(map(lambda i: str(i), skiphours), ':')
				self._commit()
			return (idx, found)
		finally: self._release()

	def _setFeedLastRun(self, url, message, ready):
		try:
			self._acquire()
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				feed = self.feeds[idx]
				feed.message = message
				feed.ready = ready
				feed.lastrun = time.time()
				self._commit()
			return (idx, found)
		finally: self._release()
	
	# this method adds or updates an item in the aggregator database
	# items are connected to a feed either by a directly passed in
	# feedurl parameter or by connection with their source/sourceurl
	# parameters. If none is given, they are not entered to the history!
	def addItem(self, item, fetchtime=None):
		try:
			self._acquire()
			feedurl = item.get('feedurl','')
			feeds = self.feeds.select({
				'title':item['source'],
				'link':item['sourceurl']
			})
			if len(feeds):
				feedurl = feeds[0].url
			if not(fetchtime):
				fetchtime = time.time()
			key = item['guid'] or item['trackbackurl']
			if not(key):
				key = md5.md5()
				key.update(item['title'])
				key.update(item['link'])
				key.update(item['desc'])
				key.update(item['pubdate'])
				key.update(item['othersource'])
				key.update(item['othersourceurl'])
				key.update(item['comments'])
				key = key.hexdigest()
			id = None
			itemidx = 0
			itemfound = 0
			(idx, found) = self.history.locate({'key':key})
			if found:
				id = self.history[idx].id
				self.history[idx].fetchtime = fetchtime
				if feedurl:
					self.history[idx].feedurl = feedurl
				(itemidx, itemfound) = self.items.locate({'id':id})
				if itemfound:
					itm = self.items[itemidx]
					itm.guid = item['guid']
					itm.title = item['title']
					itm.link = item['link']
					itm.desc = item['desc']
					itm.source = item['source']
					itm.sourceurl = item['sourceurl']
					itm.othersource = item['othersource']
					itm.othersourceurl = item['othersourceurl']
					itm.pubdate = item['pubdate']
					itm.comments = item['comments']
					itm.trackbackurl = item['trackbackurl']
					return ('update', id)
				else:
					return ('seen', id)
			else:
				self.prefs[0].maxid += 1
				item['id'] = self.prefs[0].maxid
				item['fetchtime'] = fetchtime
				self.items.append(item)
				if feedurl:
					self.history.append({
						'key':key,
						'fetchtime':fetchtime,
						'id':self.prefs[0].maxid,
						'feedurl':feedurl
					})
				return ('new', self.prefs[0].maxid)
		finally:
			self._commit()
			self._release()
	
	# this method purges the history. This sometimes is necessary if it gets broken
	# you can give a feed url if you only want to purge the history of
	# one feed.
	def purgeHistory(self, feedurl=None):
		try:
			self._acquire()
			list = range(len(self.history))
			list.reverse()
			for idx in list:
				if not(feedurl) or feedurl == self.history[idx].feedurl:
					self.history.delete(idx)
			self._commit()
		finally: self._release()
	
	# this method prunes the history for a given feed and prunes entries that are not
	# connected to any feed
	def pruneHistory(self, feedurl):
		try:
			self._acquire()
			count = 0
			# items of a feed that are older than 24 hours can be removed
			reftime = time.time() - 24*3600
			# anonymous items (from old versions) that are older than 72 hours can be removed
			reftimeold = time.time() - 72*3600
			list = range(len(self.history))
			list.reverse()
			for idx in list:
				if feedurl == self.history[idx].feedurl and self.history[idx].fetchtime < reftime:
					self.history.delete(idx)
					count += 1
				elif not(self.history[idx].feedurl) and self.history[idx].fetchtime < reftimeold:
					self.history.delete(idx)
					count += 1
			self._commit()
			if count:
				self.logVerbose(_('%d items have been deleted from the history'), count)
		finally: self._release()
	
	# this method updates feed data from the feed (and updates the feed itself)
	# the history for the given feed is pruned.
	def updateFeed(self, url):
		downstream = self.getToolByNamespace('downstream')
		macros = self.getToolByNamespace('macros')
		feed = self.getFeed(url)
		if feed:
			message = feed['message']
			ready = feed['ready']
			try:
				fp = downstream.download(url)
			except:
				message = _('There was an error when downloading the feed')
				ready = 0
				fp = None
			if fp:
				src = fp.read()
				fp.close()
				p = None
				try:
					src = self._prepareRSS(src)
					p = self._parseRSS(src)
					parseok = 1
				except:
					(e, d, tb) = sys.exc_info()
					message = _('Exception "%s: %s"') % (e, d)
					for row in traceback.extract_tb(tb):
						(file, line, function, source) = row
						message += '<br>%s[%s] in %s' % (file, line, function)
					ready = 0
					parseok = 0
				if parseok:
					if len(p.channels) == 0:
						message = _('There was no channel in the feed')
						ready = 0
					elif len(p.channels) > 1:
						message = _('There was more than one channel in the feed')
						ready = 0
					else:
						chan = p.channels[0]
						if feed['parsehook']:
							if hasattr(macros,feed['parsehook']):
								getattr(macros,feed['parsehook'])(chan)
						if chan.has_key('title'):
						
							def getValue(tag, default=''):
								tuple = chan.get(tag, [default, {}])
								return tuple[0]
							
							feedauthor = macros.stripHTML(getValue('managingeditor'))
							if feedauthor == '':
								feedauthor = macros.stripHTML(getValue('webmaster'))
							feedlink = getValue('link',url)
							if feedlink.find('<') >= 0:
								feedlink = macros.stripHTML(feedlink)
							self._setFeedData(url,
								macros.safeHTML(getValue('title')),
								feedlink,
								macros.safeHTML(getValue('description')),
								feedauthor,
								chan.get('<skiphours>',[])
							)
							feed = self.getFeed(url)
							
							message = ''
							ready = 1
							
							if chan.has_key('<items>'):
								fetchtime = int(time.time())
								items = chan['<items>']
								items.reverse()
								count = 0
								update = 0
								for item in items:
									
									def getValue(tag, default=''):
										tuple = item.get(tag)
										if tuple and type(tuple) in (type([]), type((1,1))):
											return tuple[0]
										else:
											return default
										
									title = macros.unquoteXML(getValue('title'))
									title = macros.safeHTML(title)
									link = getValue('link')
									if link.find('<') >= 0:
										link = macros.stripHTML(link)
									desc = getValue('description')
									desc = macros.unquoteXML(desc)
									try:
										desc = macros.safeHTML(desc)
									except:
										desc = macros.quoteXML(desc)
									image = getValue('<image>')
									if image:
										desc = '<img align="left" src="%s"> %s' % (
											image, desc
										)
									pubdate = macros.stripHTML(getValue('pubdate'))
									guid = macros.stripHTML(getValue('guid'))
									tburl = ''
									if item.has_key('trackback:ping'):
										tburl = macros.repeatedUnquoteXML(item['trackback:ping'][1].get('rdf:resource',''))
									if tburl.find('<') >= 0:
										tburl = macros.stripHTML(tburl)
									comments = macros.repeatedUnquoteXML(getValue('comments'))
									if comments.find('<') >= 0:
										comments = macros.stripHTML(comments)
									othersource = macros.unquoteXML(getValue('source'))
									othersource = macros.safeHTML(othersource)
									othersourceurl = ''
									if othersource:
										othersourceurl = macros.repeatedUnquoteXML(item['source'][1].get('url',''))
										if othersourceurl.find('<') >= 0:
											othersourceurl = macros.stripHTML(othersourceurl)
									
									(updateflag, id) = self.addItem({
										'guid': guid,
										'title': title,
										'link': link,
										'desc': desc,
										'source': feed['title'],
										'sourceurl': feed['link'],
										'othersource': othersource,
										'othersourceurl': othersourceurl,
										'pubdate': pubdate,
										'comments': comments,
										'feedurl': url,
										'trackbackurl': tburl
									}, fetchtime)
									if updateflag == 'new':
										count += 1
									elif updateflag == 'update':
										update += 1
										
								if count or update:
									self.log(_('stored %d and updated %d items (%d in feed %s)'),
										count, update, len(items),
										'<a href="%s">%s</a>' % (url, url)
									)
							self.pruneHistory(url)
									
						else:
							message = _('Title is missing from the channel')
			self._setFeedLastRun(url, message, ready)
			return (ready, message)
		return (0, _('the feed %s could not be found'))
	
	# this method deletes a feed from the aggregator. It deletes
	# the feed, all items in the item database and all entries
	# in the history.
	def deleteFeed(self, url):
		try:
			self._acquire()
			(idx, found) = self.feeds.locate({'url':url})
			if found:
				source = self.feeds[idx].title
				sourceurl = self.feeds[idx].link
				self.feeds.delete(idx)
				todel = []
				idx = 0
				for item in self.items:
					if item.sourceurl == sourceurl and item.source == source and not(item.locked):
						todel.append(idx)
					idx += 1
				todel.sort()
				todel.reverse()
				for idx in todel:
					self.items.delete(idx)
				self.purgeHistory(url)
				return (1, _('the feed %s was deleted successfully') % url)
			return (0, _('the feed %s could not be found') % url)
		finally:
			self._commit()
			self._release()
	
	# this method returns a preference setting
	def getPref(self, pref):
		try:
			self._acquire()
			return getattr(self.prefs[0], pref)
		finally: self._release()
			
		

__desc__ = _("This tool manages RSS newsfeeds.")

PyDS.Tool.registerTool(AggregatorTool('aggregator', _('Aggregator'), __desc__, 200))

