"""

Python Desktop Server - Default Configuration

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: DefaultConfig.py,v 1.18 2003/05/27 09:18:20 gb Exp $

import os
import sys
from distutils.sysconfig import PREFIX

def constructPath(*args):
	path = apply(os.path.join, args)
	try: os.makedirs(path)
	except: pass
	return path

# compute some special filenames and paths
try: 
	HOMEDIR = os.environ['PYDS_HOME']
except KeyError: 
	if sys.platform == 'win32': 
		try: 
			home = os.environ['HOME'] # for custom environments
		except KeyError: # more likely default NT/2K/XP install
			home = os.environ['HOMEDRIVE'] + os.environ['HOMEPATH']
	else: 
		home = os.environ['HOME']
	HOMEDIR = constructPath(home, '.PyDS')

SHAREDDIR = constructPath(PREFIX, 'share', 'pyds')
MASTERTOOLDIR = constructPath(SHAREDDIR, 'tools')
ROOTDIR = constructPath(HOMEDIR, 'www')
STATICDIR = constructPath(ROOTDIR, 'static')
IMAGESDIR = constructPath(ROOTDIR, 'images')
LOGDIR = constructPath(HOMEDIR, 'log')
VARDIR = constructPath(HOMEDIR, 'var')
ETCDIR = constructPath(HOMEDIR, 'etc')
TOOLDIR = constructPath(HOMEDIR, 'tools')
TMPLDIR = constructPath(HOMEDIR, 'templates')
FONTDIR = constructPath(SHAREDDIR, 'fonts')

PIDFILE = os.path.join(VARDIR, 'pyds.pid')

# the titletag for the desktop. Usually not changed
titletag = 'PyDS'

# how is your server called?
serverhostname = 'localhost'
serverip = '127.0.0.1'
serverport = 4334

# should the monitor server be started? If yes, set monitorport to
# something different than 0 and add a monitor password.
# Monitor will only run on serverip, not on remoteip!
monitorport = 0
monitorpassword = ''

# how is your server called when called from remote sites?
# these settings must be overwritten to enable remote accessability
remotehostname = ''
remoteip = ''
remoteport = 4334

# this is the default language
language = 'en'

# these usually are't overwritten, they are used by Server.py and restore
echoLog = 0
proxy = ''
verbose = 0

