"""

Python Desktop Server - metaWeblog API

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: MetaWeblogAPI.py,v 1.2 2003/03/17 16:33:27 gb Exp $

import sys
import time

import PyDS.Tool

class MetaWeblogAPI(PyDS.Tool.StandardTool):

	# this method creates a new posting. The blogid is ignored, as we only have
	# one blog. the struct must conform to the RSS 2.0 struct elements. The
	# following elements are used: title, link, description, category,
	# source. Other items are ignored and some might be inferred from the
	# context. The publish flag is ignored, all content is published in PyDS
	def newPost(self, blogId, username, password, struct, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 1
		if struct['description'][0] == '<':
			structured = 0
		categories = struct['category']
		if type(categories) in (type([]), type((1,1))):
			categories = list(categories)
		else:
			categories = [categories]
		id = weblog.postNewEntry(
			struct['title'],
			struct['link'],
			struct['description'],
			struct['source']['name'],
			struct['source']['url'],
			onhome=not(struct.get('flNotOnHomePage',0)),
			categories=categories,
			structured=structured)
		return id
	
	# this method edits an old posting. The blogid is ignored, as we only have
	# one blog. The struct must conform to the RSS 2.0 struct elements. The
	# following elements are used: title, link, description, category,
	# source. Other items are ignored and some might be inferred from the
	# context. The publish flag is ignored, as PyDS publishes all content.
	def editPost(self, blogId, username, password, struct, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 1
		if struct['description'][0] == '<':
			structured = 0
		categories = struct['category']
		if type(categories) in (type([]), type((1,1))):
			categories = list(categories)
		else:
			categories = [categories]
		id = weblog.editOldEntry(
			title=struct['title'],
			link=struct['link'],
			text=struct['description'],
			source=struct['source']['name'],
			sourceurl=struct['source']['url'],
			onhome=not(struct.get('flNotOnHomePage',0)),
			categories=categories,
			structured=structured)
		return 1
	
	# this method returns an existing posting in the RSS format. It only
	# returns title, link, description, category and source.
	def getPost(self, postid, username, password):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		post = weblog.getPost(postid)
		if post:
			res = {}
			if post['title']:
				res['title'] = post['title']
			if post['link']:
				res['link'] = post['link']
			if post['categories']:
				res['category'] = post['categories']
			if post['source'] and post['sourceurl']:
				res['source'] = {
					'name': post['source'],
					'url': post['sourceurl']
				}
			res['description'] = post['description']
			res['flNotOnHomePage'] = not(post['onhome'])
			return res
		else:
			raise _('Posting %s not found') % postid

	# this method returns a list of existing categories
	def getCategories(self, blogid, username, password):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		return weblog.getCategories()

__desc__ = _("This tool implements the metaWeblog API.")

PyDS.Tool.registerTool(MetaWeblogAPI('metaWeblog', _('metaWeblog API'), __desc__, 1411))

