"""

Python Desktop Server - Navigation Box Editor and Renderer

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: NavigationTool.py,v 1.20 2003/06/12 11:26:00 gb Exp $

import PyDS.Tool
import metakit

linktypelist = [
	['home', _('Link to your homepage'), 0],
	['category', _('Link to a weblog category'), 1],
	['story', _('Link to a story'), 1],
	['storyindex', _('Link to the story index'), 0],
	['estory', _('Link to an external story'), 1],
	['estoryindex', _('Link to the external story index'), 0],
	['picture', _('Link to a picture'), 1],
	['pictureindex', _('Link to the picture index'), 0],
	['macroindex', _('Link to the macro index'), 0],
	['nuggetindex', _('Link to the nugget index'), 0],
	['wiki', _('Link to a wiki page'), 0],
	['direct', _('Direct link to some url'), 1],
]
linktypes = {}
for lt in linktypelist:
	linktypes[lt[0]] = lt[1]

class NavigationTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		try:
			self._acquire()
			self.navigation= self.db.getas(
				"navigation[level:I,title:S,type:S,name:S,desc:S]"
			).ordered(2)
		finally: self._release()
	
	def _backup(self):
		return self._backupTable('navigation')

	def _initopts(self):
		self.url = self.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
	
	# this method returns description of preferences panels that are
	# hooked up into preferences by this tool
	def getPreferencePanels(self):
		return [
			['navigation_panel', _('Navigation'),
				_('You can configure your navigation links here.')]
		]
			
	def navigation_panel(self, req):
		html = '<p>%s' % _("Here you can add and delete navigation links for your homepage.")
		html += '<form action="%s" method="POST">' % req.getUrl('addnavigation_redir', tool='navigation')
		html += '<table border=0 width="100%">'
		linktitle = req.getFirstValue('linktitle')
		linktype = ''
		linkname = ''
		linkdesc = ''
		linklevel = 0
		maxlevel = 0
		hasLinkType = {}
		for link in self.navigation:
			if link.level > maxlevel:
				maxlevel = link.level
			if link.title == linktitle:
				linktype = link.type
				linkname = link.name
				linkdesc = link.desc
				linklevel = link.level
			hasLinkType[link.type] = 1
			html += '<tr><td colspan=2 class="whitebox">'
			html += '<table border=0 width="100%"><tr><td width="80%">'
			spec = linktypes[link.type]
			if not(link.type in ('home', 'storyindex', 'macroindex', 'nuggetindex', 'estoryindex')):
				spec += ': %s' % link.name
			html += '<b>%d. %s(%s):</b> %s' % (
				link.level,
				link.title,
				spec,
				link.desc
			)
			html += '</td><td width="20%" align="right" valign="top">'
			html += '%s<br>%s</td></tr>' % (
				'<a href="%s">%s</a>' % (
					req.getUrl(
						'panel_html',
						tool='preferences',
						panel='navigation_panel',
						fortool='navigation',
						linktitle=link.title
					), _('edit')
				),
				'<a href="%s">%s</a>' % (
					req.getUrl('delnavigation_redir', tool='navigation', linktitle=link.title),
					_('delete')
				)
			)
			html += '</table></td></tr>'
		if not(linklevel):
			linklevel = maxlevel+1
		html += '<tr><td>%s</td><td>' %_('link level')
		html += '<input name="linklevel" size=3 value="%d">' % linklevel
		html += '</td></tr><tr><td>%s</td><td>' %_('link title')
		html += '<input name="linktitle" size=10 value="%s">' % linktitle
		html += '</td></tr><tr><td>%s</td><td>' %_('link type')
		html += '<select name="linktype">'
		for (t, n, m) in linktypelist:
			if hasLinkType.get(t,0) and not(m) and linktype != t:
				continue 
			if t == linktype:
				html += '<option value="%s" selected>%s' % (t, n)
			else:
				html += '<option value="%s">%s' % (t, n)
		html += '</select></td></tr><tr><td>%s</td><td>' %_('link object')
		html += '<input name="linkname" size=30 value="%s">' % linkname
		html += '</td></tr><tr><td>%s</td><td>' %_('link desc')
		html += '<textarea name="linkdesc" cols=50 rows=5>%s</textarea>' % linkdesc
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="NavigationSubmit" value="%s">' % _('add')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		return html
		
	def addnavigation_redir(self, req):
		title = req.getFirstValue('linktitle')
		typ = req.getFirstValue('linktype')
		name = req.getFirstValue('linkname')
		desc = req.getFirstValue('linkdesc')
		level = int(req.getFirstValue('linklevel'))
		self.addNavigation(level, title, typ, name, desc)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
		
	def delnavigation_redir(self, req):
		title = req.getFirstValue('linktitle')
		self.delNavigation(title)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
	
	# add or change an existing navigation link
	def addNavigation(self, level, title, typ, name, desc):
		try:
			self._acquire()
			res = self.navigation.select({'title':title})
			if len(res):
				row = res[0]
				row.name = name
				row.type = typ
				row.desc = desc
				row.level = level
			else:
				self.navigation.append({'title':title, 'type':typ, 'name':name, 'desc':desc, 'level':level})
			self._commit()
			return len(res)
		finally: self._release()
		
	# delete an existing navigation link
	def delNavigation(self, title):
		try:
			self._acquire()
			todel = []
			idx = 0
			for nav in self.navigation:
				if nav.title == title:
					todel.append(idx)
				idx += 1
			todel.sort()
			todel.reverse()
			for idx in todel:
				self.navigation.delete(idx)
				self._commit()
			return len(todel)
		finally: self._release()
	
	# get the record of an existing navigation link
	def getNavigation(self, title):
		try:
			self._acquire()
			res = self.navigation.select({'title':title})
			if len(res):
				row = res[0]
				res = {
					'title':row.title,
					'type':row.type,
					'name':row.name,
					'desc':row.desc,
					'level':row.level,
				}
				return res
			return {}
		finally: self._release()

	# returns wether any navigation links are defined
	def isactive(self):
		try:
			self._acquire()
			return len(self.navigation) > 0
		finally: self._release()

	# renders the navigation block as HTML
	def render(self):
		try:
			self._acquire()
			html = '<table border="0">'
			for link in self.navigation:
				url = ''
				if link.type == 'home':
					url = self.getToolByNamespace('upstream').get('prefs','cloudurl')
				elif link.type == 'storyindex':
					url = self.getToolByNamespace('stories').getStaticUrl()
				elif link.type == 'estoryindex':
					url = self.getToolByNamespace('externals').getStaticUrl()
				elif link.type == 'macroindex':
					url = self.getToolByNamespace('macros').getStaticUrl()
				elif link.type == 'nuggetindex':
					url = self.getToolByNamespace('nuggets').getStaticUrl()
				elif link.type == 'pictureindex':
					url = self.getToolByNamespace('pictures').getStaticUrl()
				elif link.type == 'picture':
					pic = link.name
					cat = self.getToolByNamespace('pictures').getFormatList()[0]['name']
					if link.name.find(':'):
						(pic, cat) = link.name.split(':')
					url = self.getToolByNamespace('pictures').getStaticUrl(page=('%s.html' % pic), category=cat)
				elif link.type == 'category':
					url = self.getToolByNamespace('weblog').getStaticUrl(category=link.name)
				elif link.type == 'story':
					url = self.getToolByNamespace('stories').getStaticUrl(page=('%s.html' % link.name))
				elif link.type == 'wiki':
					url = self.getToolByNamespace('wiki').getStaticUrl(page=('%s.html' % link.name))
				elif link.type == 'estory':
					url = self.getToolByNamespace('externals').getStaticUrl(page=link.name)
				elif link.type == 'direct':
					url = link.name
				if url:
					html += '<tr><td>'
					html += '<a href="%s" title="%s">%s</a>' % (
						url,
						link.desc,
						link.title
					)
					html += '</td></tr>'
			html += '</table>'
			return html
		finally: self._release()


__desc__ = _("This tool allows to configure navigation links.")

PyDS.Tool.registerTool(NavigationTool('navigation', _('Navigation'), __desc__, 1100))

