"""

Python Desktop Server - Nuggets Package

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: NuggetsTool.py,v 1.16 2003/06/12 11:26:00 gb Exp $

import re
import sys
import time
import PyDS.Tool

from Cheetah.Template import Template

protectedNuggets = [
	('aboveNavigation', _('This nugget will show above the navigation'), ''),
	('belowNavigation', _('This nuget will show below the navigation'), ''),
	('firstBelowCalendar', _('This nugget will show first below the calendar'), ''),
	('secondBelowCalendar', _('This nugget will show second below the calendar'), ''),
	('customHeaders', _('This nugget will deliver custom headers'), ''),
]

class NuggetsTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		try:
			self._acquire()
			self.nuggets = self.db.getas(
				"nuggets[name:S,title:S,source:S,protected:I,publish:I,active:I]"
			).ordered(1)
			self._commit()
			for row in protectedNuggets:
				(name, title, source) = row
				if not(self._hasNugget(name)):
					self._defNugget(name, title, source, protected=1, publish=0)
		finally: self._release()
	
	def _backup(self):
		return self._backupTable('nuggets')
	
	def _initopts(self):
		try:
			self._acquire()
			self._hookIntoRenderTool(self._renderIndex)
			timer = self.getToolByNamespace('timer')
			timer._addHook('select', 'nugget', _('recreate rendered output of active nugget'), self._nuggetTimerHook)
		finally: self._release()
	
	def _renderIndex(self):
		try:
			self._acquire()
			req = self._getDummyRequest()
			html = "<ul>"
			for nugget in self.nuggets:
				if nugget.publish:
					html += '<li><b><a href="%s">%s</a>:</b> %s</li>' % (
						self.getStaticUrl(page='%s.html' % nugget.name),
						nugget.name,
						nugget.title
					)
			html += "</ul>"
			src = req.renderPage('BaseRendering',
				body=html,
				category=''
			)
			self.storeHTML(src)
		finally: self._release()
	
	def _nuggetTimerHook(self, qual, item=None):
		if qual == 'title':
			return _('recreate rendered output of active nugget')
		elif qual == 'select':
			html = '<p><select name="item">'
			try:
				self._acquire()
				for nugget in self.nuggets.select({'active':1}).sort(self.nuggets.title):
					html += '<option value="%s">%s' % (
						nugget.name,
						nugget.title
					)
			finally: self._release()
			html += '</select>'
			return html
		elif qual == 'item':
			return _('recreate rendered output of nugget "%s"') % item
		elif qual == 'run':
			try:
				self._acquire()
				(idx, found) = self.nuggets.locate({'name':item})
				if found:
					nugget = self.nuggets[idx]
					renderer = self._getStaticRenderer(nugget)
					src = renderer()
					src = src.replace("'", "\\'")
					src = src.replace("\n", "\\n")
					src = src.replace("\t", "\\t")
					src = src.replace("\r", "\\r")
					src = "document.write('%s');" % src
					self.storeFile(src, '%s.js' % item)
				else:
					raise KeyError(name)
			finally: self._release()
		else:
			raise ValueError('%s is an unknown qualifier' % qual)
	
	def _renderNugget(self, key, data):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			(idx, found) = self.nuggets.locate({'name':key})
			if found:
				nugget = self.nuggets[idx]
				html = _('You can copy and paste the source of the following nugget into your own nugget edit box.')
				html += '<pre class="whiteboxsmall">'
				html += macros.quoteXML(nugget.source)
				html += '</pre>'
				req = self._getDummyRequest()
				src = req.renderPage('BaseRendering',
					body=html,
					category='',
					nuggetid=key
				)
				self.storeHTML(src, page='%s.html' % key)
		finally: self._release()
	
	def _deleteRenderedNugget(self, key, data):
		try:
			self._acquire()
			self.deleteFile(name='%s.html' % key)
			self.deleteFile(name='%s.js' % key)
		finally: self._release()
	
	def _rerender(self):
		try:
			self._acquire()
			for nugget in self.nuggets:
				if nugget.publish:
					self.queue.append(
						self._renderNugget,
						nugget.name,
						None
					)
				else:
					self.queue.append(
						self._deleteRenderedNugget,
						nugget.name,
						None
					)
		finally: self._release()

	def _hasNugget(self, name):
		try:
			self._acquire()
			(idx, found) = self.nuggets.locate({'name':name})
			return found > 0
		finally: self._release()
	
	def _getNugget(self, name):
		try:
			self._acquire()
			(idx, found) = self.nuggets.locate({'name':name})
			if found:
				nugget = self.nuggets[idx]
				return {
					'name': nugget.name,
					'title': nugget.title,
					'source': nugget.source,
					'publish': nugget.publish,
					'active': nugget.active,
				}
			else:
				return None
		finally: self._release()

	def _delNugget(self, name):
		try:
			self._acquire()
			idx = 0
			todel = []
			for nugget in self.nuggets:
				if nugget.name == name:
					todel.append(idx)
				idx += 1
			todel.sort()
			todel.reverse()
			for idx in todel:
				self.nuggets.delete(idx)
				self._commit()
			self.queue.append(
				self._deleteRenderedNugget,
				name,
				None
			)
		finally: self._release()
		
	def _defNugget(self, name, title, source, protected=None, publish=0, active=0):
		try:
			self._acquire()
			(idx, found) = self.nuggets.locate({'name':name})
			oldpublish = 0
			if not(found):
				if protected == None:
					protected = 0
				self.nuggets.append({
					'name':name,
					'title':title,
					'source':source,
					'protected':protected,
					'publish':publish,
					'active':active
				})
			else:
				self.nuggets[idx].title = title
				self.nuggets[idx].source = source
				self.nuggets[idx].publish = publish
				self.nuggets[idx].active = active
				if protected != None:
					self.nuggets[idx].protected = protected
			self._commit()
			if active:
				self._nuggetTimerHook('run', name)
			if publish:
				self.queue.append(
					self._renderNugget,
					name,
					None
				)
			else:
				if oldpublish:
					self.queue.append(
						self._deleteRenderedNugget,
						name,
						None
					)
					
		finally: self._release()
		
	def __getattr__(self, name):
		try:
			self._acquire()
			(idx, found) = self.nuggets.locate({'name':name})
			if found:
				nugget = self.nuggets[idx]
				if nugget.active:
					return self._getDynamicRenderer(nugget)
				else:
					return self._getStaticRenderer(nugget)
			else:
				raise KeyError(name)
		finally: self._release()
	
	def _getDynamicRenderer(self, nugget):
		try:
			self._acquire()
			html = '<script src="%s" type="text/javascript">' % self.getStaticUrl(page='%s.js' % nugget.name)
			html += '</script>'
			return lambda **kw: html
		finally: self._release()
	
	def _getStaticRenderer(self, nugget):

		def mergeDict(dict, key, value):
			dict = dict.copy()
			dict[key] = value
			return dict

		def renderNugget(src, *args):
			return str(Template(src, searchList=args))

		try:
			self._acquire()
			kw2 = self._getDummyRequest().getNamespace()
			return lambda **kw: renderNugget(nugget.source, kw, mergeDict(kw2, 'context', kw), PyDS.Tool._ToolNamespace)
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			statics = self.nuggets.select({'active':0})
			html += _('<li>There are %s active nuggets in the database</li>') % (len(self.nuggets) - len(statics))
			html += _('<li>There are %s static nuggets in the database</li>') % len(statics)
			html += _('<li>There are %s nuggets in the render queue</li>') % self.queue.length()
			html += '</ul>'
			return html
		finally: self._release()

	# this method returns the page title for rendering
	def getPageTitle(self, context):
		if context.has_key('nuggetid'):
			nugget = self._getNugget(context['nuggetid'])
			return nugget['name']
		else:
			return _('All published nuggets')

	# this method returns the page description for rendering
	def getPageDescription(self, context):
		if context.has_key('nuggetid'):
			nugget = self._getNugget(context['nuggetid'])
			return nugget['title']
		else:
			return PyDS.Tool.StandardTool.getPageDescription(self, context)

	def index_html(self, req):
		html = _("Nuggets are small building blocks that are rendered into the source when called. They get passed namespaces just like templates.")
		t = _('Click to add a new nugget')
		html = '<p><a href="%s" title="%s">%s</a>' % (
			self.getUrl('edit_html'), t,t
		)
		html += "<ul>"
		try:
			self._acquire()
			for nugget in self.nuggets:
					etext = ''
					html += '<li>%s<b><a href="%s">%s</a>:</b> %s</li>' % (
						etext,
						self.getUrl('edit_html', name=nugget.name),
						nugget.name,
						nugget.title
					)
		finally: self._release()
		html += "</ul>"
		req.setLocalValue('title', _("All defined nuggets"))
		return req.renderPage('BaseTemplate', body=html)
		
	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		name = req.getFirstValue('name')
		title = ''
		source = '<div class="whiteboxsmall">\n</div>'
		protected = 1
		publish = 0
		active = 0
		try:
			self._acquire()
			(idx, found) = self.nuggets.locate({'name':name})
			if found:
				n = self.nuggets[idx]
				title = n.title
				source = n.source
				protected = n.protected
				publish = n.publish
				active = n.active
		finally: self._release()
		html = '<pre class="whiteboxsmall">%s</pre>' % macros.quoteXML(source)
		html += '<form action="%s" method="POST">' % self.getUrl('edit_redir')
		html += '<table border=0>'
		html += '<tr>'
		html += '<th align="right">%s</th><td><input name="name" value="%s" size=50></td>' % (
			_('Name'), name
		)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><input name="title" value="%s" size=50></td>' % (
			_('Title'), title
		)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Publish')
		html += '<select name="publish">'
		if publish:
			html += '<option value="1" selected>%s' % _('yes')
			html += '<option value="0">%s' % _('no')
		else:
			html += '<option value="1">%s' % _('yes')
			html += '<option value="0" selected>%s' % _('no')
		html += '</select>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Active')
		html += '<select name="active">'
		if active:
			html += '<option value="1" selected>%s' % _('yes')
			html += '<option value="0">%s' % _('no')
		else:
			html += '<option value="1">%s' % _('yes')
			html += '<option value="0" selected>%s' % _('no')
		html += '</select>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><textarea nowrap name="source" rows=10 cols=60>%s</textarea></td>' % (
			_('Source'), source
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		if not(protected):
			html += '<input type=submit name="NuggetsDelete" value="%s">' % _('delete')
		html += ' <input type=submit name="NuggetsSubmit" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different nugget')
		)
		req.setLocalValue('title', name)
		return req.renderPage('BaseTemplate', body=html)
		
	def edit_redir(self, req):
		name = req.getFirstValue('name')
		if req.getFirstValue('NuggetsDelete'):
			self._delNugget(name)
			return self.getUrl()
		else:
			title = req.getFirstValue('title')
			source = req.getFirstValue('source')
			publish = int(req.getFirstValue('publish'))
			active = int(req.getFirstValue('active'))
			if title and source:
				source = source.replace('\r\n','\n')
				source = source.replace('\r', '\n')
				if source[-1:] != '\n':
					source += '\n'
				self._defNugget(name, title, source, publish=publish, active=active)
				return self.getUrl('edit_html', name=name)
			return self.getUrl('edit_html', name=name, errmsg=_('You need to give title and source!'))
		

__desc__ = _("This is a tool that manages nuggets for text rendering.")

PyDS.Tool.registerTool(NuggetsTool('nuggets', _('Nuggets'), __desc__, 640))

