"""

Python Desktop Server - Preferences Editor

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: PreferencesTool.py,v 1.26 2003/06/12 15:35:13 gb Exp $

import PyDS.Tool
import PyDS.Translation
import re
import metakit
import time
import string

PREFS = [
	[('private', _('Private Data'), _('These are preferences about you and your weblog.')),
		('name', 'S30', _('Your Name')),
		('email', 'S30', _('Your eMail')),
		('title', 'S30', _('Weblog Title')),
		('description', 'T30,5', _('Weblog Description')),
		('radiocompat', 'Y', _('Radio compatible URLs')),
		('structured', 'Y', _('Use structured text as default')),
		('upstreambackup', 'Y', _('Upstream backup files to cloud')),
		('fullrss', 'Y', _('Put always full text into RSS feeds')),
		('suppara', 'Y', _('Suppress single paragraph tags')),
	],
	[('RPC', _('XML-RPC Configuration'), _('These are preferences that allow you to set up remote procedure call availability to your server')),
		('enabled', 'Y', _('enable RPC')),
		('username', 'S20', _('Username')),
		('password', 'P20', _('Password')),
	],
]

globalname = ''
globalemail = ''
globaltitle = ''

def cacheName(nv, ov):
	global globalname
	globalname = nv

def cacheEMail(nv, ov):
	global globalemail
	globalemail = nv

def cacheTitle(nv, ov):
	global globaltitle
	globaltitle = nv

class PreferencesTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		try:
			self._acquire()
			self.prefhash = {}
			self.prefs = {}
			for pb in PREFS:
				self.prefhash[pb[0][0]] = pb[1:]
				dbstr = "%s[" % pb[0][0]
				liste = []
				for p in pb[1:]:
					if type(p[1]) == type([]):
						liste.append('%s:S' % p[0])
					elif p[1][0] in ('T','P'):
						liste.append('%s:S' % p[0])
					elif p[1][0] == 'Y':
						liste.append('%s:I' % p[0])
					else:
						liste.append('%s:%s' % (p[0],p[1][0]))
				dbstr += string.join(liste, ',') + "]"
				self.prefs[pb[0][0]] = self.db.getas(dbstr)
				if len(self.prefs[pb[0][0]]) == 0:
					self.prefs[pb[0][0]].append()
					self._commit()
		finally: self._release()
	
	def _restoreGetTable(self, tablename):
		if self.prefs.has_key(tablename):
			return self.prefs[tablename]
		return None
	
	def _backup(self):
		l = []
		for pb in self.prefs.keys():
			if pb != 'RPC':
				l.extend(self._backupTable(pb, tableobj=self.prefs[pb], exceptcols=['password']))
		return l

	def _initopts(self):
		global globalname, globalemail, globaltitle
		try:
			self._acquire()
			PyDS.Tool.StandardTool._initopts(self)
			self.hooks = {}
			self.installPreferencesHook('private', 'name', cacheName)
			self.installPreferencesHook('private', 'email', cacheEMail)
			self.installPreferencesHook('private', 'title', cacheTitle)
			cacheName(self.get('private', 'name'), globalname)
			cacheEMail(self.get('private', 'email'), globalemail)
			cacheTitle(self.get('private', 'title'), globaltitle)
		finally: self._release()

	# this method tells the server that some mandatory preferences
	# are not given
	def needsStuff(self, req):
		if globalname == '' or globalemail == '' or globaltitle == '':
			return self.getUrl(
				page='edit_html',
				pb='private',
				errmsg=_('You need at least to give Name, eMail and Title')
			)
		else:
			return 0
			
	def index_html(self, req):
		html = _('<p>Choose below the group of preferences you want to change')
		html += '<table border=0 width="100%">'
		for pb in PREFS:
			html += '<tr><td>'
			html += '<a href="%s?pb=%s">%s</a>: %s' % (
				req.getUrl('edit_html'),
				pb[0][0], pb[0][1], pb[0][2]
			)
			html += '</td></tr>'
		html += '<tr><td>&nbsp;</td></tr>'
		for tool in PyDS.Tool.getTools():
			if hasattr(tool, 'getPreferencePanels') and tool.level < 2000:
				for panel in tool.getPreferencePanels():
					html += '<tr><td>'
					html += '<a href="%s">%s</a>: %s' % (
						req.getUrl(
							'panel_html',
							panel=panel[0],
							fortool=tool.name
						), panel[1], panel[2]
					)
					html += '</td></tr>'
		html += '</table>'
		req.setLocalValue('title', _('Preferences'))
		return req.renderPage('BaseTemplate', body=html)
		
	def panel_html(self, req):
		panel = req.getFirstValue('panel')
		tool = self.getToolByNamespace(req.getFirstValue('fortool'))
		html = getattr(tool, panel)(req)
		html += '<p align="right"><a href="%s">%s</a></p>' % (req.getUrl(), _('choose another group'))
		req.setLocalValue('title', _('Preferences for "%s"') % tool.name)
		return req.renderPage('BaseTemplate', body=html)

	def edit_html(self, req, pb=None):
		try:
			self._acquire()
			pref = req.getFirstValue('pb', pb)
			if pref and self.prefhash.has_key(pref):
				html = _('<p>Edit your settings below and press submit')
				html += '<form action="%s" method="POST">' % req.getUrl('edit_redir')
				html += '<table border=0">'
				for p in self.prefhash[pref]:
					html += '<tr>'
					html += '<th align="left">%s</th>' % p[2]
					pv = getattr(self.prefs[pref][0], p[0])
					if type(p[1]) == type([]):
						html += '<td align="left"><select name="%s">' % p[0]
						for o in p[1]:
							if pv == o[0]:
								html += '<option value="%s" selected>%s' % (o[0], o[1])
							else:
								html += '<option value="%s">%s' % (o[0], o[1])
						html += '</select></td>'
					elif p[1][0] == 'Y':
						html += '<td align="left"><select name="%s">' % p[0]
						if pv:
							html += '<option value="1" selected>%s' % _('yes')
						else:
							html += '<option value="1">%s' % _('yes')
						if pv:
							html += '<option value="0">%s' % _('no')
						else:
							html += '<option value="0" selected>%s' % _('no')
						html += '</select></td>'
					elif p[1][0] == 'S':
						width = 50
						if len(p[1]) > 1:
							width = int(p[1][1:])
						html += '<td align="left"><input name="%s" value="%s" size=%d></td>' % (p[0], pv, width)
					elif p[1][0] == 'P':
						width = 30
						if len(p[1]) > 1:
							width = int(p[1][1:])
						html += '<td align="left"><input type="password" name="%s" value="%s" size=%d></td>' % (p[0], pv, width)
					elif p[1][0] == 'I':
						html += '<td align="left"><input name="%s" value="%d" size=10></td>' % (p[0], pv)
					elif p[1][0] == 'T':
						cols = 50
						rows = 10
						if len(p[1]) > 1:
							s = p[1][1:]
							if s.find(',')>=0:
								(c,r) = s.split(',')
								cols = int(c)
								rows = int(r)
							else:
								cols = int(s)
						html += '<td align="left"><textarea name=%s cols=%d rows=%d>%s</textarea></td>' % (p[0], cols, rows, pv)
					html += '</tr>'
				html += '<tr><td colspan=2 align="right">'
				html += '<input type=submit name=PreferencesSubmit value="%s">' % _('Submit Changes')
				html += '</td></tr>'
				html += '</table>'
				html += '<input type="hidden" name="pb" value="%s">' % pref
				html += '</form>'
				html += '<p align="right"><a href="%s">%s</a></p>' % (req.getUrl(), _('choose another group'))
				req.setLocalValue('title', _('Preferences'))
				return req.renderPage('BaseTemplate', body=html)
			else:
				return self.index_html(req)
		finally: self._release()

	def edit_redir(self, req):
		try:
			self._acquire()
			pref = req.getFirstValue('pb')
			if pref and self.prefhash.has_key(pref):
				for p in self.prefhash[pref]:
					pf = req.getFirstValue(p[0])
					if type(p[1]) == type([]):
						pass
					elif p[1] in ('I', 'Y'):
						pf = int(pf)
					if self.hooks.has_key(pref) and self.hooks[pref].has_key(p[0]):
						for hook in self.hooks[pref][p[0]]:
							hook(pf, getattr(self.prefs[pref][0], p[0]))
					setattr(self.prefs[pref][0], p[0], pf)
				self._commit()
				return req.getUrl('edit_html', pb=pref)
			else:
				return req.getUrl()
		finally: self._release()
	
	# preference hooks are hooked up to a special field and called with
	# newval and oldval. This method usually is called internally
	def installPreferencesHook(self, pb, pf, func):
		try:
			self._acquire()
			if not(self.hooks.has_key(pb)):
				self.hooks[pb] = {}
			if not(self.hooks[pb].has_key(pf)):
				self.hooks[pb][pf] =[]
			self.hooks[pb][pf].append(func)
		finally: self._release()

	# get the value of a preference in a preference block (pb)
	def get(self, pb, pref):
		try:
			self._acquire()
			if self.prefs.has_key(pb):
				if hasattr(self.prefs[pb][0], pref):
					return getattr(self.prefs[pb][0], pref)
				else:
					return ''
			else:
				return ''
		finally: self._release()
	
	# get the value of a proference in a preference block, but quote
	# all chars that are critically in XML
	def getXML(self, pb, pref):
		macros = self.getToolByNamespace('macros')
		res = self.get(pb,pref)
		if res:
			return macros.quoteXML(res)
		else:
			return ''


__desc__ = _("This tool allows to edit your preferences.")

PyDS.Tool.registerTool(PreferencesTool('preferences', _('Preferences'), __desc__, 700))

