# -*- Mode: Python -*-

# soap_handler is a SOAP handler for medusa and PyDS built on xmlrpc_handler
#
# this adaption was done by Georg Bauer <bauer@gws.ms>

# See http://www.xml-rpc.com/
#     http://www.pythonware.com/products/xmlrpc/

# Based on "xmlrpcserver.py" by Fredrik Lundh (fredrik@pythonware.com)

# $Id: SOAPHandler.py,v 1.13 2003/05/22 10:36:55 gb Exp $

import string
import sys
import traceback
import SOAPpy

from medusa import http_server, xmlrpc_handler

from PyDS.Tool import _ToolNamespace, log

# patch a new characters method into SOAPParser that converts to
# iso-8859-1. This is a hack, but SOAPpy doesn't support passing encodings
# to the parser and PyDS internal stuff doesn't work well with unicode.
def characters(self, c):
	""" adds chars from a string only in latin-1 encoding """
	if self._data != None:
		self._data += c.encode('iso-8859-1')

SOAPpy.SOAPParser.characters = characters

# patch a new dump_string into SOAPBuilder for the same reason
def dump_string(self, obj, tag, typed = 0, ns_map = {}):
	""" delivers chars from a string only in latin-1 encoding """
	import cgi

	tag = tag or self.gentag()

	id = self.checkref(obj, tag, ns_map)
	if id == None:
		return
	
	try: data = obj._marshalData()
	except: data = obj

	if type(data) == type(u''):
		data = data.decode('iso-8859-1')
	
	self.out.append(self.dumper(None, "string", cgi.escape(data), tag,
		typed, ns_map, self.genroot(ns_map), id))

SOAPpy.SOAPBuilder.dump_string = dump_string
SOAPpy.SOAPBuilder.dump_str = dump_string
SOAPpy.SOAPBuilder.dump_unicode = dump_string

class soap_handler(xmlrpc_handler.xmlrpc_handler):

	def match (self, request):
		prefs = _ToolNamespace['preferences']
		if prefs.get('RPC','enabled'):
			if request.uri[:5] == '/SOAP':
				return 1
			else:
				return 0
		else:
			return 0

	def continue_request (self, data, request):
		(r, header, body, attrs) = SOAPpy.parseSOAPRPC(
			data, header=1, body=1, attrs=1
		)

		method = r._name
		args = r._aslist
		kw = r._asdict

		path, params, query, fragmet = request.split_uri()
		if path[:6] == '/SOAP/':
			method = '%s.%s' % (path[6:], method)

		ns = r._ns
		nsmethod = method
		if ns:
			nsmethod = "%s:%s" % (ns, method)

		try:
		    # generate response
			try:
				fr = self.call (method, args, kw)
			except:
				# report exception back to server
				(e, d, tb) = sys.exc_info()
				f = SOAPpy.faultType('%s:Client' % SOAPpy.NS.ENV_T, e)
				f._setDetail(d)
				response = SOAPpy.buildSOAP(f, encoding = 'iso-8859-1')
				msg = _('XML-RPC call exception %s: %s') % (e, d)
				for row in traceback.extract_tb(tb):
					(file, line, function, source) = row
					msg += '<br>%s[%s] in %s' % (file, line, function)
				if type(msg) == type(u''):
					msg = msg.encode('iso-8859-1')
				log(msg)
			else:
				   response = SOAPpy.buildSOAP(
					kw = {'%sResponse' % method: fr},
					encoding = 'iso-8859-1'
				)
		except:
			# internal error, report as HTTP server error
			(e, d, tb) = sys.exc_info()
			f = SOAPpy.faultType('%s:Server' % SOAPpy.NS.ENV_T, e)
			f._setDetail(d)
			response = SOAPpy.buildSOAP(f, encoding = 'iso-8859-1')
			msg = _('XML-RPC call exception %s: %s') % (e, d)
			for row in traceback.extract_tb(tb):
				(file, line, function, source) = row
				msg += '<br>%s[%s] in %s' % (file, line, function)
			log(msg)

		# got a valid XML RPC response
		request['Content-Type'] = 'text/xml'
		request.push (response)
		request.done()

	def call (self, method, args, kw):
		prefs = _ToolNamespace['preferences']
		(base, method) = method.split('.')

		authuser = ''

		if method != 'listRegisteredFunctions' and not(base in ['blogger', 'metaWeblog']):
			if len(args) < 2:
				raise _("Too few arguments - need at least user and password"), len(args)
			if prefs.get('RPC', 'username') != args[0]:
				raise _("Authentication failure"), args[0]
			if prefs.get('RPC', 'password') != args[1]:
				raise _("Authentication failure"), args[0]
			authuser = args[0]
			args = args[2:]

		if _ToolNamespace.has_key(base):
			tool = _ToolNamespace[base]
			try:
				_flet.begin(rpc=1, current=tool, authenticated_user=authuser)
				if hasattr(tool, method):
					meth = getattr(tool, method)
					return apply(meth, args)
				else:
					raise str(_("Method not found in namespace %s") % base), str(method)
			finally:
				_flet.end()
		else:
			raise _("Namespace not found"), base

