"""

Python Desktop Server - Stories Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: StoryTool.py,v 1.41 2003/06/15 16:52:55 gb Exp $

import time
import sys
import traceback

import PyDS.Tool

class StoryTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		try:
			self._acquire()
			self.stories = self.db.getas(
				"stories[id:I,pubtime:F,title:S,desc:S,text:S,rendered:S,structured:I,draft:I,renderhook:S,quiet:I]"
			).ordered(1)
			self.prefs = self.db.getas(
				"prefs[maxid:I,title:S,desc:S]"
			)
			if len(self.prefs) == 0:
				self.prefs.append({'maxid':0})
			self._commit()
		finally: self._release()
	
	def _backup(self):
		l = self._backupTable('prefs')
		l.extend(self._backupTable('stories', sortcols=['id']))
		return l

	def _initopts(self):
		try:
			self._acquire()
			self._hookIntoRenderTool(self._renderStoryIndexOnline)
			self.rss = 1
			timer = self.getToolByNamespace('timer')
			timer._addHook('select', 'story', _('recreate rendered story text'), self._storyTimerHook)
		finally: self._release()

	def _getNewId(self):
		try:
			self._acquire()
			self.prefs[0].maxid += 1
			self._commit()
			return self.prefs[0].maxid
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d stories in the database</li>') % len(self.stories)
			html += _('<li>There are %d stories in the render queue</li>') % self.queue.length()
			html += '</ul>'
			return html
		finally: self._release()

	def _storyTimerHook(self, qual, item=None):
		if qual == 'title':
			return _('recreate rendered story text')
		elif qual == 'select':
			html = '<p><select name="item">'
			try:
				self._acquire()
				for story in self.stories:
					html += '<option value="%s">%s' % (
						story.id,
						story.title
					)
				html += '</select>'
			finally: self._release()
			return html
		elif qual == 'item':
			if type(item) != type(1):
				item = int(item)
			item = self.getStory(item)
			return _('recreate rendered content of story "%s"') % item['title']
		elif qual == 'run':
			if type(item) != type(1):
				item = int(item)
			self.processStory(item)
		else:
			raise ValueError('%s is an unknown qualifier' % qual)
	
	# this method returns the title of a page for rendering
	def getPageTitle(self, context):
		try:
			self._acquire()
			title = PyDS.Tool.StandardTool.getPageTitle(self, context)
			if self.prefs[0].title:
				title = self.prefs[0].title
			if context.get('storyid',''):
				story = self.getStory(int(context['storyid']))
				if story:
					title = story['title']
			return title
		finally: self._release()

	# this method returns the title and description of a page for rendering
	def getPageDescription(self, context):
		try:
			self._acquire()
			desc = PyDS.Tool.StandardTool.getPageDescription(self, context)
			if self.prefs[0].desc:
				desc = self.prefs[0].desc
			if context.get('storyid',''):
				story = self.getStory(int(context['storyid']))
				if story:
					desc = story['desc']
			return desc
		finally: self._release()
	
	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		t = _('Click to add a new story')
		html = '<p><a href="%s" title="%s">%s</a>' % (
			self.getUrl('edit_html'), t, t
		)
		html += "<ul>"
		try:
			self._acquire()
			for story in self.stories:
				draftimg = ''
				if story.draft:
					draftimg = '<img src="/images/new.gif" alt="%s">' % _('this story is in draft mode')
				html += '<li>%s<b><a href="%s">%s (%d)</a>:</b> %s <span class="smallfont">(%s %s)</span></li>' % (
					draftimg,
					self.getUrl('show_html', id=str(story.id)),
					story.title,
					story.id,
					story.desc,
					_('last change'),
					macros.Today(story.pubtime, withtime=1),
				)
		finally: self._release()
		html += "</ul>"
		req.setLocalValue('title', self.getPageTitle({}))
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def show_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = int(req.getFirstValue('id'))
		story = self.getStory(id)
		html = '<p class="whitebox">%s</p>' % story['desc']
		html += '<p><a href="%s">%s</a></p>' % (
			self.getUrl('edit_html', id=str(id)),
			_('Click here to change this story')
		)
		html += '<p><a href="%s">%s</a></p>' % (
			self.getUrl('index_html',
				tool='weblog',
				ptitle=story['title'],
				plink=self.getStaticUrl(page=('%d.html' % id)),
				ptext=story['desc']
			),
			_('Click here to post this story')
		)
		html += req.renderTextWithTraceback(
			story['text'],
			structured=story['structured']
		)
		html += '<p align="right" class="smallfont">%s %s</p>' % (
			_('last change'),
			macros.Today(story['pubtime'], withtime=1)
		)
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different story')
		)
		req.setLocalValue('title', story['title'])
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		if id:
			id = int(id)
		else:
			id = -1
		story = self.getStory(id)
		title = _('place your story title here')
		text = _('place your story text here')
		desc = _('place a short description here')
		draft = 0
		quiet = 0
		renderhook = ''
		structured = self.getPreferencesTool().get('private','structured')
		if story:
			title = story['title']
			text = story['text']
			desc = story['desc']
			structured = story['structured']
			draft = story['draft']
			quiet = story['quiet']
			renderhook = story['renderhook']
		html = '<form action="%s" method="POST">' % self.getUrl('edit_redir')
		if story:
			html += '<input type="hidden" name="id" value="%d">' % id
		html += '<table border=0>'
		html += '<tr>'
		if story:
			html += '<th align="right">%s</th><td> %d</td>' % (_('Story'), id)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><input name="title" value="%s" size=50></td>' % (
			_('Title'), title
		)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Draft')
		html += '<select name="draft">'
		if draft:
			html += '<option value="1" selected>%s' % _('yes')
			html += '<option value="0">%s' % _('no')
		else:
			html += '<option value="1">%s' % _('yes')
			html += '<option value="0" selected>%s' % _('no')
		html += '</select>'
		html += '</td>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _("Don't list in RSS")
		html += '<select name="quiet">'
		if quiet:
			html += '<option value="1" selected>%s' % _('yes')
			html += '<option value="0">%s' % _('no')
		else:
			html += '<option value="1">%s' % _('yes')
			html += '<option value="0" selected>%s' % _('no')
		html += '</select>'
		html += '</td>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Textformat')
		html += '<select name="structured">'
		if structured:
			html += '<option value="1" selected>%s' % _('structured text')
			html += '<option value="0">%s' % _('HTML source')
		else:
			html += '<option value="1">%s' % _('structured text')
			html += '<option value="0" selected>%s' % _('HTML source')
		html += '</select></td>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('run on rendering')
		html += '<select name="renderhook">'
		html += '<option value="">%s' % _('no macro selected')
		for macro in macros._getMacros():
			if renderhook == macro['name']:
				html += '<option value="%s" selected>%s' % (
					macro['name'],
					macro['name']
				)
			else:
				html += '<option value="%s">%s' % (
					macro['name'],
					macro['name']
				)
		html += '</select></td>'
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="desc" rows=5 cols=60>%s</textarea></td>' % (
			_('Description'), desc
		)
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="text" rows=30 cols=60>%s</textarea></td>' % (
			_('Text'), text
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="StoryDelete" value="%s">' % _('delete')
		html += ' <input type=submit name="StorySave" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different story')
		)
		req.setLocalValue('title', title)
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def edit_redir(self, req):
		id = req.getFirstValue('id')
		if req.getFirstValue('StoryDelete'):
			self.deleteStory(int(id))
			return self.getUrl()
		else:
			if id:
				id = int(id)
			else:
				id = -1
			title = req.getFirstValue('title')
			text = req.getFirstValue('text')
			desc = req.getFirstValue('desc')
			structured = int(req.getFirstValue('structured'))
			draft = int(req.getFirstValue('draft'))
			quiet = int(req.getFirstValue('quiet'))
			renderhook = req.getFirstValue('renderhook')
			if title and text:
				id = self.addStory(id, title, text, desc, structured, draft, quiet, renderhook)
				return self.getUrl('show_html', id=str(id))
			else:
				if id:
					return self.getUrl('edit_html', id=str(id), errmsg=_('You need to give title and text!'))
				else:
					return self.getUrl('edit_html', errmsg=_('You need to give title and text!'))

	def getPreferencePanels(self):
		return [
			['stories_panel', _('Stories'),
				_('Here you can configure the title and description of the story index page.')]
		]
			
	def stories_panel(self, req):
		try:
			self._acquire()
			html = '<p>%s' % _('Here you can configure the title and description of the story index page.')
			html += '<form action="%s" method="POST">' % req.getUrl('stories_redir', tool='stories')
			html += '<table border=0 width="100%">'
			html += '<tr><td>%s</td><td>' %_('title')
			html += '<input name="title" size=50 value="%s">' % self.prefs[0].title
			html += '</td></tr><tr><td>%s</td><td>' % _('description')
			html += '<textarea name="desc" rows=5 cols=50>%s</textarea>' % self.prefs[0].desc
			html += '</td></tr><tr><td colspan=2 align="right">'
			html += '<input type=submit name="StoriesSubmit" value="%s">' % _('save')
			html += '</td></tr>'
			html += '</table>'
			html += '</form>'
			return html
		finally: self._release()
		
	def stories_redir(self, req):
		title = req.getFirstValue('title')
		desc = req.getFirstValue('desc')
		try:
			self._acquire()
			self.prefs[0].title = title
			self.prefs[0].desc = desc
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='stories_panel',
				fortool='stories'
			)
		finally: self._release()
		
	def _renderStoryOnline(self, id, dummy):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			(idx, found) = self.stories.locate({'id':id})
			if found:
				story = self.stories[idx]
				req = self._getDummyRequest(now=story.pubtime)
				html = story.rendered
				html += '<p align="right" class="smallfont">%s %s</p>' % (
					_('last change'),
					macros.Today(story.pubtime, withtime=1)
				)
				src = req.renderPage(
					'BaseRendering',
					storyid = id,
					body = html,
					category = ''
				)
				self.storeHTML(src, page='%d.html' % id)
		finally: self._release()

	# this method puts a render request into the queue
	def renderStory(self, id):
		story = self.getStory(id)
		if story['draft']:
			self.queue.append(self._deleteRenderedStory, id, None)
		else:
			self.queue.append(self._renderStoryOnline, id, None)
		return story['draft']

	def _rerender(self):
		try:
			self._acquire()
			for story in self.stories:
				self.renderStory(story.id)
		finally: self._release()

	def _renderStoryIndexOnline(self):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			req = self._getDummyRequest()
			html = "<ul>"
			for story in self.stories:
				if not(story.draft):
					html += '<li><a href="%s">%s</a> <span class="smallfont">(%s %s)</span></li>' % (
						self.getStaticUrl(page=('%d.html' % story.id)),
						story.title,
						_('last change'),
						macros.Today(story.pubtime, withtime=1)
					)
			html += "</ul>"
			src = req.renderPage(
				'BaseRendering',
				body = html,
				category = ''
			)
			self.storeHTML(src)
			xml = req.renderPage('RSSFeedRendering', category='')
			self.storeXML(xml, category='')
		finally: self._release()
	
	def _deleteRenderedStory(self, id, dummy):
		self.deleteFile('%d.html' % id)

	# this method deletes an existing story and drops the rendered output
	def deleteStory(self, id):
		try:
			self._acquire()
			(idx, found) = self.stories.locate({'id':id})
			if found:
				self.stories.delete(idx)
				self._commit()
				self.queue.append(
					self._deleteRenderedStory,
					id,
					None
				)
			return found
		finally: self._release()

	# this method processes the story text and recreates the rendered
	# cache
	def processStory(self, id):
		macros = self.getToolByNamespace('macros')
		hook = ''
		try:
			self._acquire()
			(idx, found) = self.stories.locate({'id':id})
			if found:
				story = self.stories[idx]
				req = self._getDummyRequest()
				rendered = req.renderText(
					story.text,
					structured=story.structured
				)
				if story.rendered != rendered:
					hook = story.renderhook
					story.rendered = rendered
					story.pubtime = time.time()
					self._commit()
					self.renderStory(id)
			return idx >= 0
		finally:
			self._release()
			if hook and hasattr(macros, hook):
				getattr(macros, hook)(id)
	
	# this methods adds a new story
	def addStory(self, id, title, text, desc, structured, draft, quiet, renderhook):
		macros = self.getToolByNamespace('macros')
		hook = ''
		try:
			self._acquire()
			req = self._getDummyRequest()
			rendered = req.renderText(text, structured=structured)
			pubtime = time.time()
			(idx, found) = self.stories.locate({'id':id})
			if found:
				story = self.stories[idx]
				if story.title != title or story.text != text or story.rendered != rendered or story.desc != desc or story.draft != draft or story.quiet != quiet or story.renderhook != renderhook:
					story.title = title
					story.text = text
					story.pubtime = pubtime
					story.rendered = rendered
					story.desc = desc
					story.structured = structured
					story.draft = draft
					story.quiet = quiet
					story.renderhook = renderhook
					hook = renderhook
					self._commit()
					self.renderStory(id)
			else:
				id = self._getNewId()
				self.stories.append({
					'id': id,
					'title': title,
					'text': text,
					'pubtime': pubtime,
					'rendered': rendered,
					'desc': desc,
					'structured': structured,
					'draft': draft,
					'quiet': quiet,
					'renderhook': renderhook
				})
				hook = renderhook
				self._commit()
				self.renderStory(id)
			return id
		finally:
			self._release()
			if hook and hasattr(macros, hook):
				getattr(macros, hook)(id)
	
	# this method returns a story as a dictionary
	def getStory(self, id):
		try:
			self._acquire()
			id = int(id)
			(idx, found) = self.stories.locate({'id':id})
			if found:
				story = self.stories[idx]
				return {
					'id': id,
					'title': story.title,
					'text': story.text,
					'rendered': story.rendered,
					'pubtime': int(story.pubtime),
					'desc': story.desc,
					'structured': story.structured,
					'draft': story.draft,
					'quiet': story.quiet,
					'renderhook': story.renderhook
				}
			else:
				return None
		finally: self._release()
	
	# this method returns the URL for a story depending on context
	def getStoryUrl(self, id):
		if _flet.desktop:
			return self.getUrl('show_html', id=str(id))
		else:
			return self.getStaticUrl(page='%s.html' % id)
		
	# tools wanting to catch an unresolved StructuredText link should return a 
	# CSS class and cloud URL from this method
	def getCloudUrlMatchingStrayLink(self, shortcut): 
		shortcut = shortcut.lower()
		try:
			self._acquire()
			for story in self.stories: 
				if (story.title.lower().find(shortcut) >= 0 or str(story.id) == shortcut) and not(story.draft):
					break 
			else: # no match
				return ()
			return ('story', self.getStoryUrl(story.id))
		finally:
			self._release()
		
	# this method returns the title of a page for rendering. This does a
	# this method returns the title of the RSS feed
	def getRSSTitle(self, category=''):
		try:
			self._acquire()
			prefs = self.getPreferencesTool()
			title = prefs.get('private', 'title')
			if self.prefs[0].title:
				title = '%s: %s' % (title, self.prefs[0].title)
			else:
				title = '%s: %s' % (title, _('Stories'))
			if not(title):
				title = _('An Unconfigured Python Desktop Server: Stories')
			return title
		finally: self._release()
	
	# this method returns the description of the RSS feed
	def getRSSDescription(self, category=''):
		try:
			self._acquire()
			prefs = self.getPreferencesTool()
			desc = prefs.get('private', 'description')
			if self.prefs[0].desc:
				desc = self.prefs[0].desc
			return desc
		finally: self._release()
	
	# this method returns the stories as an item list, orderd by time
	def getRSSItems(self, category=''):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			items = []
			for story in self.stories:
				if not(story.draft) and not(story.quiet):
					item = {
						'title': macros.quoteXML(story.title),
						'link': self.getStaticUrl(page='%d.html' % story.id),
						'guid': self.getStaticUrl(page='%d.html' % story.id),
						'pubdate': macros.TodayFull(now=story.pubtime),
						'pubtime': story.pubtime,
						'source': '',
						'sourceurl': '',
						'comments': '',
						'trackback': '',
						'description': macros.quoteXML(story.desc)
					}
					items.append(item)
			items.sort(lambda a,b: -1*cmp(a['pubtime'],b['pubtime']))
			if len(items) > 20:
				items = items[:21]
			for item in items:
				del item['pubtime']
			return items
		finally: self._release()

	# this method returns hours without stories
	def getRSSSkipHours(self, category=''):
		try:
			self._acquire()
			hours = {}
			for story in self.stories:
				hours[time.gmtime(story.pubtime)[3]] = 1
		finally: self._release()
		nonhours = []
		for hour in range(24):
			if not(hours.has_key(hour)):
				nonhours.append(hour)
		return nonhours


__desc__ = _("This is a tool that manages longer stories.")

PyDS.Tool.registerTool(StoryTool('stories', _('Stories'), __desc__, 100))

