"""

Python Desktop Server - Trackback client

Copyright (c) 2003, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: TrackbackTool.py,v 1.4 2003/04/13 16:33:15 gb Exp $

import re
import time
import urllib

import PyDS.Tool

class TrackbackTool(PyDS.Tool.StandardTool):

	def _initopts(self):
		weblog = self.getToolByNamespace('weblog')
		weblog._addHook('edit', 'select', _('Trackback'), self._trackbackPingForm, self._trackbackActive)
		weblog._addHook('editold', 'ping', '', self._trackbackPingHook, self._trackbackActive)
		weblog._addHook('postnew', 'ping', '', self._trackbackPingHook, self._trackbackActive)

	def _initthread(self, server):
		while server.isRunning():
			time.sleep(10)
			if self.queue.length():
				while self.queue.length():
					(jobf, key, data) = self.queue.shift()
					jobf(key, data)
				self.logVerbose(_('work done, going to sleep again'))

	def _trackbackActive(self, *args, **kw):
		return 1
	
	def _trackbackPingForm(self, req, *args, **kw):
		html = '<input name="tburl" size=40 value="%s">' % req.getFirstValue('ptburl')
		return html

	def _trackbackPingHook(self, tool, req, id, *args, **kw):
		tburl = req.getFirstValue('tburl')
		if tburl:
			self.queue.append(
				self._trackbackPing,
				(tburl, tool.name, id),
				None
			)
	
	def _trackbackPing(self, key, data):
		(url, toolname, id) = key
		tool = self.getToolByNamespace(toolname)
		res = self.trackbackPing(url, tool, id)
		if res:
			tool.log(_('Trackback error: %s'), res)

	def trackbackPing(self, url, tool, id):

		def stripHTML(str):
			macros = self.getToolByNamespace('macros')
			str = macros.unquoteXML(str)
			return macros.stripHTML(str)

		item = tool.getRSSItem(id, fullrss=1)
		excerpt = stripHTML(item['description'])
		excerpt = re.sub('\s+', ' ', excerpt)
		if len(excerpt) > 250:
			excerpt = excerpt[:250]
			while len(excerpt) > 200 and excerpt[-1] != ' ':
				excerpt = excerpt[:-1]
			excerpt += ' ...'
		data = urllib.urlencode({
			'blog_name': stripHTML(tool.getRSSTitle()),
			'title': stripHTML(item['title']),
			'url': item['guid'],
			'excerpt': excerpt
		})
		downstream = self.getToolByNamespace('downstream')
		opener = downstream.getUrlOpener(1)
		xml = opener.open(url, data).read()
		if xml[:5] == '<?xml':
			m1 = re.match(r'.*<response>.*<error>(.*?)</error>.*</response>.*', xml, re.DOTALL)
			if m1:
				if m1.group(1).strip() == '0':
					return ''
				else:
					m2 = re.match(r'.*<response>.*<message>(.*?)</message>.*</response>.*', xml, re.DOTALL)
					if m2:
						return m2.group(1)
					else:
						return _('unspecified error in trackback ping')
			else:
				return _('No trackback response found')
		else:
			return _('No XML format found in trackback response')
	
	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d pings in the queue</li>') % self.queue.length()
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
__desc__ = _("This tool implements trackback client suppport.")

PyDS.Tool.registerTool(TrackbackTool('trackback', _('Trackback Ping'), __desc__, 1310))

