"""

Python Desktop Server - Wiki Editor

Copyright (c) 2002, Garth T Kidd <garth@deadlybloodyserious.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: WikiTool.py,v 1.7 2003/06/12 11:26:00 gb Exp $

import PyDS.Tool
import metakit
import time
import string
import sys
import cgi
import re
import os.path

def _munge(s): 
	"Elminiate non-alphanumeric and make lowercase."
	t = re.sub('\W', '', s)
	return t.lower()

class WikiTool(PyDS.Tool.StandardTool):

	def _recachepostings(self):
		raise NotImplementedError # can't see a need for it

	def _initdb(self):
		try:
			self._acquire()
			self.nodes = self.db.getas(
				"nodes[id:S,title:S,text:S,pubdate:S,pubtime:F,categoryid:S,baseid:S]"
			).ordered(1)
			self.categories = self.db.getas(
				"categories[id:S]"
			).ordered(1)
			self._commit()
		finally: self._release()

	
	def _backup(self):
		l = self._backupTable('nodes')
		return l
			
	def _initopts(self):
		self.rss = 1
		if self._getNode('index') is None: 
			self.addNode('index', 
			             _("Wiki Index"), 
				     _("Edit this page to begin your Wiki"))
		if self._getNode('404') is None: 
			self.addNode('404', 
			             _("Unresolved Target"), 
				     _("Not-yet-resolved links in the Wiki are pointed here."))
		
	def _initthread(self, server):
		while server.isRunning():
			time.sleep(10)
			if self.queue.length():
				while self.queue.length():
					(jobf, key, data) = self.queue.shift()
					jobf(key, data)
				# since we had something to render, we should
				# now render the RSS feed to get a current one
				req = self._getDummyRequest()
				xml = req.renderPage('RSSFeedRendering', category='')
				self.storeXML(xml, category='')
				self.logVerbose(_('work done, going to sleep again'))
	
	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d wiki nodes in the database</li>') % len(self.nodes)
			html += _('<li>There are %d items in the render queue</li>') % self.queue.length()
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
	def _search(self, query):
		return [] # not implemented yet

	# this method returns an URL to the cloud in the context of the current tool
	# or the given tool. Weblog patches the URL if radio compatibility is requested.
	def getStaticUrl(self, page=None, tool=None, year=None, month=None, day=None, category='', base=None, flat=0):
		if filter(None, (year, month, day, category)): 
			pass # aah, ignore it for the time being 
			# raise NotImplementedError, _("argument not supported by Wiki tool")
		if page is None: 
			pass # aah, ignore it for the time being
			# raise ValueError, _("page argument required by Wiki tool")
		url = PyDS.Tool.StandardTool.getStaticUrl(self, page, tool, year, month, day, category, base, flat)
		return url

	# This method returns the URL for a node. Depending on context it
	# returns a desktop URL or a cloud URL.
	def getCloudUrlForNode(self, id): 
		if _flet.desktop:
			return self.getUrl('show_html', id=str(id))
		else:
			node = self.getNode(id)
			if node:
				return self.getStaticUrl(page='%s.html' % id)
			else:
				return self.getStaticUrl(page='404.html')
		
	# tools wanting to catch an unresolved StructuredText link should return a 
	# CSS class and cloud URL from this method
	def getCloudUrlMatchingStrayLink(self, shortcut): 
		lowershortcut = shortcut.lower()
		mungedshortcut = _munge(shortcut)
		gcufn = self.getCloudUrlForNode
		rownumsByLowerid = {}
		rownumsByMungedTitle = {}
		try:
			self._acquire()
			rownum = 0
			for node in self.nodes: 
				rownumsByLowerid[node.id.lower()] = rownum
				rownumsByMungedTitle[_munge(node.title)] = rownum
				rownum += 1
			res = rownumsByLowerid.get(lowershortcut)
			if res is None: # try subnodes
				try: 
					sub = _flet.id.lower()+'/'+lowershortcut
					res = rownumsByLowerid.get(sub)
				except KeyError: 
					pass
			if res is None: # try peers in matching categoryid
				try: 
					inSame = _flet.categoryid+'/'+lowershortcut
					res = rownumsByLowerid.get(inSame)
				except KeyError: 
					pass
			if res is None: # try titles
				res = rownumsByMungedTitle.get(mungedshortcut)
			if res is None: # try titles again
				res = rownumsByLowerid.get(mungedshortcut)
			if res is None: # give up
				return ('undefinednode', gcufn(mungedshortcut))
			# victory!
			return ('node', gcufn(self.nodes[res].id))
		finally:
			self._release()
		
	# this method returns the title of a page for rendering. This does a
	# lookup of data in the category or in the preferences and adds
	# date specifiers. The context passed in is the context that you
	# get in your template.
	def getPageTitle(self, context):
		if context.has_key('title'): 
			return context['title']
		log('WikiTool.getPageTitle failed to find title; context.keys==[%s]', ', '.join(map(repr, context.keys())))
		title = PyDS.Tool.StandardTool.getPageTitle(self, context)
		if context.get('category',''):
			cat = self.getCategory(context['category'])
			if cat:
				title = cat['title']
		if context.get('year',0) and context.get('month',0) and context.get('day',0):
			title += ' %d.%d.%d' % (context['day'], context['month'], context['year'])
		elif context.get('year',0) and context.get('month',0):
			title += ' %d/%d' % (context['year'], context['month'])
		return title
		
	# this method returns the description of a page for rendering. It's
	# a sister to the getPageTitle method.
	def getPageDescription(self, context):
		desc = PyDS.Tool.StandardTool.getPageDescription(self, context)
		if context.get('category',''):
			cat = self.getCategory(context['category'])
			if cat:
				desc = cat['desc']
		return desc
		
	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = """\
<form action="%(edit_html)s" method="GET">
<input type="submit" value="%(editlabel)s" accesskey="n">
</form>"""  % {
			'edit_html': self.getUrl('edit_html', id=str(id)), 
			'editlabel': _("New")
		}
		html += "<dl>"
		try:
			self._acquire()
			for node in self.nodes:
				html += '<dt><a href="%s">%s (%s)</a>:</dt><dd>%s<br><span class="smallfont">(%s %s)</span></dd>' % (
					self.getUrl('show_html', id=str(node.id)),
					node.title,
					node.id,
					node.text[0:80],
					_('last change'),
					macros.Today(node.pubtime, withtime=1),
				)
		finally: self._release()
		html += "</ul>"
		req.setLocalValue('title', _('Your Wiki Nodes'))
		return req.renderPage('BaseTemplate', body=html)

	def show_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		node = self.getNode(id)
		try:
			_flet.begin(categoryid=node.get('categoryid',''), id=id)
			rendered = req.renderText(node.get('text',_('Edit the page content here.')), structured=1)
		finally: _flet.end()
		html = """\
%(rendered)s
<form action="%(mux_redir)s" method="POST">
<input type="hidden" name="id" value="%(id)s">
<input type="submit" name="WikiEdit" value="%(editlabel)s" accesskey="e">
<input type="submit" name="WikiDelete" value="%(deletelabel)s">
<input type="submit" name="WikiView" value="%(viewlabel)s" accesskey="v">
</form>
<p align="right" class="smallfont">%(lastlabel)s %(pubtimestr)s</p>
<p align="right"><a href="%(toolurl)s" accesskey="i">%(index)s</a></p>""" % {
			'title': node.get('title',_('Set your page title here')), 
			'mux_redir': self.getUrl('mux_redir'),
			'id': str(id), 
			'editlabel': _("Edit"), 
			'viewlabel': _("View"), 
			'deletelabel': _("Delete"), 
			'rendered': rendered, 
			'lastlabel': _("last change"), 
			'pubtimestr': macros.Today(node.get('pubtime',time.time()), withtime=1), 
			'toolurl': self.getUrl(), 
			'viewurl': self.getStaticUrl(page='%s.html' % id),
			'index': _("Wiki index"),
		}
		req.setLocalValue('title', node.get('title',_('Set your page title here')))
		return req.renderPage('BaseTemplate', body=html, title="Wiki: Show: %s" % node.get('title', '(no title)'))
		
	def mux_redir(self, req): 
		id = req.getFirstValue('id')
		if not id: 
			id = ''
		if req.getFirstValue('WikiEdit'): 
			return self.getUrl('edit_html', id=str(id))
		if req.getFirstValue('WikiDelete'): 
			return self.getUrl('delete_html', id=str(id))
		if req.getFirstValue('WikiView'): 
			return self.getStaticUrl(page='%s.html'%id)
		return self.getUrl('show_html', id=str(id))

	def delete_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		node = self.getNode(id)
		html = """\
%(rendered)s
<form action="%(delete_redir)s" method="POST">
<input type="hidden" name="id" value="%(id)s">
<input type="submit" name="WikiDeleteConfirm" value="%(confirmlabel)s" accesskey="d">
<input type="submit" name="WikiDeleteCancel" value="%(cancellabel)s" accesskey="c">
</form>
<p align="right" class="smallfont">%(lastlabel)s %(pubtimestr)s</p>""" % {
			'title': node['title'], 
			'show_html': self.getUrl('show_html', id=str(id)), 
			'delete_redir': self.getUrl('delete_redir', id=str(id)), 
			'id': str(id), 
			'cancellabel': _("Cancel"), 
			'confirmlabel': _("Confirm Deletion"), 
			'rendered': req.renderText(node['text'], structured=1, categoryid=node['categoryid'], id=id), 
			'lastlabel': _("last change"), 
			'pubtimestr': macros.Today(node['pubtime'], withtime=1), 
		}
		req.setLocalValue('title', node['title'])
		return req.renderPage('BaseTemplate', body=html, title="Wiki: Delete: %s" % node.get('title', '(no title)'))

	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		if req.getFirstValue('WikiView'): 
			return self.getStaticUrl(page='%s.html' % id)
		if not id:
			id = ''
		node = self.getNode(id)
		values = {} 
		for field in ['id', 'title', 'text']: 
			value = ''
			if id: # defaults
				value = id
			if node:
				value = cgi.escape(node[field])
			values[field+'value'] = value
		for field in ['Id', 'Title', 'Text', 'delete', 'save']: 
			values[field.lower()+'label'] = _(field)
		values['edit_redir'] = self.getUrl('edit_redir')
		html = """
<form action="%(edit_redir)s" method="POST">
<table border=0>
<tr>
  <th align="right">%(idlabel)s</th>
  <td><input name="oldid" value="%(idvalue)s" type="hidden">
      <input name="id" value="%(idvalue)s" size=50></td>
</tr><tr>
  <th align="right">%(titlelabel)s</th>
  <td><input name="title" value="%(titlevalue)s" size=50></td>
</tr><tr>
  <th valign="top" align="right">%(textlabel)s</th>
  <td><textarea nowrap name="text" rows=15 cols=60>%(textvalue)s</textarea></td>
</tr><tr>
  <td colspan=2 align="right">
    <input type=submit name="WikiDelete" value="%(deletelabel)s">
    <input type=submit name="WikiSave" accesskey="s" value="%(savelabel)s">
  </td>
</tr></table></form>
""" % values
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different node')
		)
		req.setLocalValue('title', _('Edit your node below'))
		return req.renderPage('BaseTemplate', body=html, title="Wiki: Edit: %s" % node.get('title', '(no title)'))

	def delete_redir(self, req):
		id = req.getFirstValue('id')
		if req.getFirstValue('WikiDeleteCancel'): 
			return self.getUrl('show_html', id=str(id))
		self.deleteNode(id)
		return self.getUrl()

	def edit_redir(self, req):
		id = req.getFirstValue('id')
		if req.getFirstValue('WikiDelete'):
			return self.getUrl('delete_html', id=str(id))
		else:
			if not id:
				id = ''
			title = req.getFirstValue('title')
			text = req.getFirstValue('text')
			desc = req.getFirstValue('desc')
			if id and text:
				id = self.addNode(id, title, text)
				return self.getUrl('show_html', id=str(id))
			else:
				if id:
					return self.getUrl('edit_html', id=str(id), errmsg=_('You need to give id and text!'))
				else:
					return self.getUrl('edit_html', errmsg=_('You need to give id and text!'))

	# this method deletes an existing node and drops the rendered output
	def deleteNode(self, id):
		try:
			self._acquire()
			(idx, found) = self.nodes.locate({'id':id})
			if found:
				self.nodes.delete(idx)
				self._commit()
				self.queue.append(
					self._deleteRenderedNode,
					id,
					None
				)
			return found
		finally: self._release()
		self._rerender() # ... and then append a whole bunch of rerenders

	def _deleteRenderedNode(self, id, dummy):
		self.deleteFile('%s.html' % id)

	# This method delivers the description of your preferences panels
	# to hook up into the preferences list.
	def getPreferencePanels(self):
		return [] # not implemented
			
	# Internal getNode; returns the row, not a dict.
	def _getNode(self, id): 
		try:
			self._acquire()
			(res,found) = self.nodes.locate({'id':id})
			if found:
				return self.nodes[res]
			else:
				return None
		finally: self._release()

	# get a posting by ID
	def getNode(self, id):
		node = self._getNode(id)
		if node is None: 
			return {}
		else: 
			return {
				'id': node.id,
				'title': node.title,
				'text': node.text,
				'pubtime': node.pubtime,
				'categoryid': node.categoryid, 
				'baseid': node.baseid, 
			}

	def addNode(self, id, title, text): 
		categoryid, baseid = os.path.split(id)
		macros = self.getToolByNamespace('macros')
		hook = ''
		try:
			self._acquire()
			req = self._getDummyRequest()
			pubtime = time.time()
			(idx, found) = self.nodes.locate({'id':id})
			if found:
				node = self.nodes[idx]
				if node.title != title or node.text != text or node.categoryid != categoryid or node.baseid != baseid:
					node.title = title
					node.text = text
					node.pubtime = pubtime
					node.categoryid = categoryid
					node.baseid = baseid
					self._commit()
					self._rerender() # might have resolved some
					# self.renderNode(id)
			else:
				self.nodes.append({
					'id': id,
					'title': title,
					'text': text,
					'pubtime': pubtime,
					'baseid': baseid,
					'categoryid': categoryid
				})
				self._commit()
				self._rerender() # might have resolved some
				# self.renderNode(id)
			return id
		finally:
			self._release()
			if hook and hasattr(macros, hook):
				getattr(macros, hook)(id)

	def _rerender(self):
		try:
			self._acquire()
			count = 0
			for node in self.nodes:
				self.renderNode(node.id)
				count += 1
			return count
		finally: self._release()

	# this method renders a single page in the background.
	def renderNode(self, id):
		self.queue.append(self.renderNodeOnline, (id,), None)
		return self.queue.length()

	# this method renders a single page, but does it online (not queued)
	def renderNodeOnline(self, key, data):
		(id,) = key
		node = self.getNode(id=id)
		pagename = '%s.html' % id
		req = self._getDummyRequest()
		toolname = self.name # consider ''
		body = req.renderText(node['text'], structured=1)
		html = req.renderPage('BaseRendering', body=body, category='', title=node.get('title', '(no title)'))
		fn = self.storeHTML(html, page=pagename, toolname=toolname)
		return fn

	# this method returns the title of the RSS feed
	def getRSSTitle(self, category=''):
		try:
			self._acquire()
			prefs = self.getPreferencesTool()
			title = prefs.get('private', 'title')
			if title:
				title += ': '
				title += _('Wiki')
			else:
				title = _('An Unconfigured Python Desktop Server: Wiki')
			return title
		finally: self._release()
	
	# this method returns the description of the RSS feed
	def getRSSDescription(self, category=''):
		try:
			self._acquire()
			prefs = self.getPreferencesTool()
			desc = prefs.get('private', 'description')
			return desc
		finally: self._release()
	
	# this method returns the stories as an item list, orderd by time
	def getRSSItems(self, category=''):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			items = []
			for node in self.nodes:
				desc = node.text
				if len(desc) > 250:
					desc = desc[:250] + ' ...'
				item = {
					'title': macros.quoteXML(node.title),
					'link': self.getCloudUrlForNode(node.id),
					'guid': self.getCloudUrlForNode(node.id),
					'pubdate': macros.TodayFull(now=node.pubtime),
					'pubtime': node.pubtime,
					'source': '',
					'sourceurl': '',
					'comments': '',
					'trackback': '',
					'description': macros.quoteXML(desc)
				}
				items.append(item)
			items.sort(lambda a,b: -1*cmp(a['pubtime'],b['pubtime']))
			if len(items) > 20:
				items = items[:21]
			for item in items:
				del item['pubtime']
			return items
		finally: self._release()

	# this method returns hours without stories
	def getRSSSkipHours(self, category=''):
		try:
			self._acquire()
			hours = {}
			for node in self.nodes:
				hours[time.gmtime(node.pubtime)[3]] = 1
		finally: self._release()
		nonhours = []
		for hour in range(24):
			if not(hours.has_key(hour)):
				nonhours.append(hour)
		return nonhours


__desc__ = _("This tool allows to edit wiki postings.")

PyDS.Tool.registerTool(WikiTool('wiki', _('Wiki'), __desc__, 150))

